%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Gloptlab.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function Gloptlab(res)
%
% main function, opens the graphical interface of Gloptlab.
%
% res           optional resolution setting,
%               0 : 800x600, 1: 1024:768, 2: 1280x800, 3:1280x1024

function Gloptlab(res)

%set path
path(gloptpath);

glbasedir = fileparts(which('Gloptlab.m'));     
%LINUX: installation tasks
if ~ispc && ~exist([glbasedir,filesep,'Gloptlab_unix.cfg'],'file')
        
    glsystem(['chmod +x ' glbasedir '/Source/Tools/Misc/execute.sh']);
    glsystem(['chmod +x ' glbasedir '/Source/Tools/External/copy_dag2gloptlab']);
    glsystem(['chmod +x ' glbasedir '/Source/Tools/External/dag2def']);
    glsystem(['chmod +x ' glbasedir '/Source/Tools/External/mod2def']);                
        
    if (exist([glbasedir,filesep,'Lib'],'dir')) % full version
        mex -setup
        
        % LP_SOLVE: compile matlab interface mex file
        lpsolvemexdir = [glbasedir,filesep,'Lib',filesep,'lp_solve_5.5',filesep,'extra',filesep,'MATLAB',filesep,'lpsolve'];
        if exist(lpsolvemexdir,'dir')
            cd(lpsolvemexdir);
            Makefile('',0,0);
            cd(glbasedir);
        end
    
        % SDPT: 
        sdptmexdir = [glbasedir,filesep,'Lib',filesep,'SDPT3-4.0-beta'];
        if exist(sdptmexdir,'dir')
            cd(sdptmexdir);
            Installmex;
            cd(glbasedir);
        end
        
    end
    
    % set lib paths, regardless whether they exist or not, this will be
    % checked later in glinitconfig.m
    glreplaceinfile('Gloptlab_unix_sample.cfg',{'%GLBASE%'},{glbasedir},'Gloptlab_unix.cfg');    
end



%get resolution
if nargin == 0, res=get(0,'screensize');res=res(3:4);
elseif res==0,  res(1)=800; res(2)=600;
elseif res==1,  res(1)=1024;res(2)=768;
elseif res==2,  res(1)=1280;res(2)=800;
elseif res==3,  res(1)=1280;res(2)=1024;
elseif res==4,  res(1)=1440;res(2)=900;
elseif res==5,  res(1)=1920;res(2)=1200;
else error(['Unsupported screen resolution mode: ' num2str(res) '!']);
end

%get gui position and size
if res(1)==800 && res(2)==600,       pos=[0.05 0.15 0.8 0.75];
elseif res(1)==1024 && res(2)==600,  pos=[0.05 0.15 0.8 0.75];
elseif res(1)==1024 && res(2)==768,  pos=[0.05 0.1 0.8 0.8];
elseif res(1)==1280 && res(2)==800,  pos=[0.05 0.1 0.8 0.8];
elseif res(1)==1280 && res(2)==1024, pos=[0.05 0.1 0.8 0.8];
elseif res(1)==1440 && res(2)==900,  pos=[0.05 0.1 0.8 0.8];
elseif res(1)==1600 && res(2)==1200, pos=[0.05 0.1 0.8 0.8];    
elseif res(1)==1680 && res(2)==1050, pos=[0.05 0.1 0.8 0.8];
elseif res(1)==1920 && res(2)==1200, pos=[0.05 0.1 0.8 0.8];
else
    pos=[0.05 0.1 0.8 0.8];
    disp(['WARNING: Unsupported screen resolution: ' num2str(res(1))...
        ' x ' num2str(res(2)) '!']);
end

%start gui
disp(['Gloptlab interface with resolution ' num2str(res(1)) ' x '...
    num2str(res(2)) ' started...']);
glmain_gui('Position',pos);
