%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmain_gui.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function varargout = glmain_gui(varargin)
%
% GUI function matching the figure glmain_gui.fig, partially created
% by using the GUIDE gui editor.
% Handles all gui interactions of the main graphical interface of Gloptlab.
%
% varargin      different arguments which alter the size, behavior etc.
%               of the gui. For detailed information see the matlab help
%               about 'figure proprties'.
%               currently the atrribute 'Position' can be set.
% varargout     hold no information for this gui

function varargout = glmain_gui(varargin)

    % Begin initialization code - DO NOT EDIT
    gui_Singleton = 1;
    gui_State = struct('gui_Name',       mfilename, ...
                       'gui_Singleton',  gui_Singleton, ...
                       'gui_OpeningFcn', @glmain_gui_OpeningFcn, ...
                       'gui_OutputFcn',  @glmain_gui_OutputFcn, ...
                       'gui_LayoutFcn',  [] , ...
                       'gui_Callback',   []);
    if nargin && ischar(varargin{1})
        gui_State.gui_Callback = str2func(varargin{1});
    end

    if nargout
      [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
    else
      gui_mainfcn(gui_State, varargin{:});
    end
    % End initialization code - DO NOT EDIT

%--------------------------------------------------------------------------
% Opening function
%--------------------------------------------------------------------------
% executes when the gui is started
function glmain_gui_OpeningFcn(hObject, eventdata, handles, varargin)
    
    %global configuration variable
    global glconf;
    global glconfstat;
     
    %init
    if isunix, configfile='Gloptlab_unix.cfg';  %default unix config file
    else configfile='Gloptlab_win.cfg';  %default windows config file
    end
    
    %set position or config file if supplied
    if nargin > 3
        for i=1:2:length(varargin)
            if strcmp(varargin{i},'Position')
                %get(hObject,'Position')
                %set(hObject,'Position',varargin{i+1});
            elseif strcmp(varargin{i},'Configfile')
                configfile=varargin{i+1};
            end
        end
    end
    
    if strcmp(get(hObject,'Visible'),'off')
        % updata the data files by adding possibly new entries
        glupdateguidata(handles);
        % set the diverse data for the gui components 
        glgetguidata(handles);
    end
    
    %set actual conffile name
    set(handles.conffile_ET,'String',configfile);
    
    %initialize the handles structure
    handles=init_handles(handles);
     
    %load configuration
    handles=loadconf(configfile, handles);
    
    %set the problem direcctory
    if isfield(glconf,'probdir') && ~isempty(glconf.probdir) && ...
            exist(glconf.probdir,'dir')==7
        %restore last session
        d=glconf.probdir;
    else
        %set the defaults
        d=glconf.workingdir;
    end
    cd (d);
    disp(['Directory changed to the working directory: ' d]);
    
    %set the selected problems
    if isfield(glconf,'selprobs') && ~isempty(glconf.selprobs)
        ind=[];
        for i=1:length(glconf.selprobs)
            if exist([d filesep glconf.selprobs{i}],'file')~=2
                %file not found, memorize index
                ind=[ind i];
            end
        end
        %remove non existing files
        glconf.selprobs(ind)=[];
        %set selected problems
        set(handles.prob_list_LB, 'String', glconf.selprobs);
    end
    
    %set the gui position
    for i=1:2:length(varargin)
        if strcmp(varargin{i},'Position')
            set(hObject,'Units','normalized','Position',varargin{i+1});
            break
        end
    end
    set(hObject,'Units','pixels');
    glconf.mainfig=hObject;
    
    %init output function, brief diary active, full diary inactive
    global fdiaryh;fdiaryh=cell(0);
    global bdiaryh;bdiaryh=handles.diary_LB;
    
    % init the graphics output
    global gloutput;gloutput=[];
    gloutput.plotfigure=gcf;
    gloutput.plotspace=get(handles.plot_P,'Position');
    
    % solver status
    status={'General Status:'};
    if glconfstat.intlab_running, status=[status;{'Intlab...running.'}];
    else status=[status;{'Intlab...missing!'}];
    end
    if glconfstat.fmincon_ok, status=[status;{'Fmincon...ready.'}];
    else status=[status;{'Fmincon...missing!'}];
    end
    if glconfstat.lpsolve_ok, status=[status;{'LpSolve...ready.'}];
    else status=[status;{'LpSolve...missing!'}];
    end
    if glconfstat.sedumi_ok, status=[status;{'SeDuMi...ready.'}];
    else status=[status;{'SeDuMi...missing!'}];
    end
    if glconfstat.sdpt3_ok, status=[status;{'SDPT3...ready.'}];
    else status=[status;{'SDPT3...missing!'}];
    end
    % config check
    miss=glchkconf();
    if ~isempty(miss)
        s='';for i=1:length(miss), s=[s miss{i} ', '];end;s=s(1:end-2);
        status=[status;{['Config...not up to date - missing fields: ' s]}];
    else
        status=[status;{'Config...ok.'}];
    end    
    diary_full_out(status);
    diary_brief_out(status);
    
    % load strategy
    %set the starting strategy file name
    if isfield(glconf, 'strategy') && ~isempty(glconf.strategy)
        name=glconf.strategy;
    else
        name='mystrategy';
    end
    handles=load_strategy(handles, name);
    
    %set new empty startegy
    %handles=clear_strategy(handles);
    
    %get directory listing
    handles=get_directory(handles);
    
    %load logo
    axes(handles.logo_AX);load gllogo;image(gllogo);
    axis off;
    
    %initialize output
    handles.output = [];
   
    glsound('start');
   
    
    if glconfstat.intlab_running, status='Ready.';
    else status='Intlab is missing, please check the configuration!';
    end
    diary_full_out(status);
    diary_brief_out(status);
        
    % Update handles structure
    guidata(hObject, handles);
        
    % UIWAIT makes glmain_gui wait for user response (see UIRESUME)
    % uiwait(handles.figure1);

%--------------------------------------------------------------------------
% Output function
%-------------------------------------------------------------------------- 
% output function of the gui, currently return nothing useful
function varargout = glmain_gui_OutputFcn(hObject, eventdata, handles)
	
    varargout{1} = handles.output;
    
%--------------------------------------------------------------------------
% Close Request function
%--------------------------------------------------------------------------
% close the gui with the exit sound
function figure1_CloseRequestFcn(hObject, eventdata, handles)
    
    if isempty(handles)
        delete(hObject); return;
    end
    
    confname=get(handles.conffile_ET,'String');
    stratname=get(handles.strategy_file_ET, 'String');
    if isempty(fileparts(confname))
        confname=which(confname);
    end
    glupdate(confname, 'strategy', stratname);
    probdir=get(handles.current_dir_ET, 'String');
    glupdate(confname, 'probdir', probdir);
    selprobs=get(handles.prob_list_LB, 'String');
    glupdate(confname, 'selprobs', selprobs);
    glsound('exit');
    delete(hObject);

%--------------------------------------------------------------------------
% Handles initialization
%--------------------------------------------------------------------------
% initialize the handles structure
function handles=init_handles(handles)
    handles.sdir='';            %selected problem directory (used in 
                                %current_dir_ET_Callback&dirs_LB_Callback)
    handles.tasks=cell(0);      %tasks cell
    handles.certs=cell(0);      %certificate cell array
    handles.random=cell(0);     %random problems
    handles.oldaxes=[];         %old axe ranges (for zooming out)
    handles.semidir=1;          %previous used direction of semidef(intera)
    
%--------------------------------------------------------------------------
% Diary and debug
%--------------------------------------------------------------------------
% full diary mode on/off toggle button callback
function fdiary_TB_Callback(hObject, eventdata, handles)
    if get(hObject, 'Value'), setdiarymode(handles, 'full');
    else setdiarymode(handles, 'brief');
    end
    
% brief diary mode on/off toggle button callback
function bdiary_TB_Callback(hObject, eventdata, handles)
    if get(hObject, 'Value'), setdiarymode(handles, 'brief');
    else setdiarymode(handles, 'full');
    end
    
% write all text from the diary to the console
function toconPB_Callback(hObject, eventdata, handles)
    txt=get(handles.diary_LB, 'String');
    for i=1:length(txt)
        disp(txt{i});
    end

% set the diary mode to mode='full' or mode='brief'       
function setdiarymode(handles, mode)
    global fdiaryh;%full diary output handle or storage cell
    global bdiaryh;%brief diary output handle or storage cell
    
    if strcmp(mode,'full')
        %store brief diary in bdiaryh
        bdiaryh=get(handles.diary_LB, 'String');
        %print full diary storage to the diary
        set(handles.diary_LB, 'String', fdiaryh,...
            'Value', max(length(fdiaryh),1));
        %set mode buttons
        set(handles.fdiary_TB,'Value',1);set(handles.bdiary_TB,'Value',0);
        %set fdiaryh as the handle of the diary
        fdiaryh=handles.diary_LB;
    elseif strcmp(mode,'brief')
        %store full diary in fdiaryh
        fdiaryh=get(handles.diary_LB, 'String');
        %print brief diary storage to the diary
        set(handles.diary_LB, 'String', bdiaryh,...
            'Value', max(length(bdiaryh),1));
        %set mode buttons
        set(handles.fdiary_TB,'Value',0);set(handles.bdiary_TB,'Value',1);
        %set bdiaryh as the handle of the diary
        bdiaryh=handles.diary_LB;
    else error('glmain:wrong diary mode!');
    end
    drawnow;
    
% diary output enable on/off callback
function diary_TB_Callback(hObject, eventdata, handles)
    if get(hObject, 'Value')
        %diary has been turned on
        set(hObject, 'String', 'Turn Off');
        set(handles.diary_T,'String','Diary: ON');
        for i=1:length(handles.tasks)
            if isfield(handles.tasks{i}, 'argin') && ...
                    isfield(handles.tasks{i}.argin,'prt') && ...
                    ~handles.tasks{i}.argin.prt,
                handles.tasks{i}.argin.prt=true;
            end
        end
    else
        %diary has been turned off
        set(hObject, 'String', 'Turn On');
        set(handles.diary_T,'String','Diary: OFF');
        for i=1:length(handles.tasks)
            if isfield(handles.tasks{i}, 'argin') && ...
                    isfield(handles.tasks{i}.argin,'prt') && ...
                    handles.tasks{i}.argin.prt,
                handles.tasks{i}.argin.prt=false;
            end
        end
    end
    handles=refresh_tasklist(handles, get(handles.task_list_LB,'Value'));
    % Update handles structure
    guidata(hObject, handles);

% append a text to the full diary
function diary_full_out(txt)
    global fdiaryh;     %full diary output handle or storage cell
    if ishandle(fdiaryh)
        %full diary mode is active set output at the handle fdiaryh
        text=get(fdiaryh, 'String');
        if iscell(txt), text=[text;txt];
        else text=[text;{txt}];
        end
        ind=max(length(text),1);
        set(fdiaryh, 'String', text, 'Value', ind);
    else
        %brief diary mode, store output in the cell fdiaryh
        if iscell(txt),fdiaryh=[fdiaryh;txt];
        else fdiaryh=[fdiaryh;{txt}];
        end
    end
    drawnow;
    
% append a text to the brief diary
function diary_brief_out(txt)
    global bdiaryh;     %brief diary output handle or storage cell
    if ishandle(bdiaryh)
        %brief diary mode is active set output at the handle fdiaryh
        text=get(bdiaryh, 'String');
        if iscell(txt), text=[text;txt];
        else text=[text;{txt}];
        end
        ind=max(length(text),1);
        set(bdiaryh, 'String', text, 'Value', ind);
    else
        %full diary mode, store output in the cell bdiaryh
        if iscell(txt),bdiaryh=[bdiaryh;txt];
        else bdiaryh=[bdiaryh;{txt}];
        end
    end
    drawnow;
    
% clear diary callback
function clear_diary_PB_Callback(hObject, eventdata, handles)
    clear_diary(handles);
    
% clear diary function
function clear_diary(handles)
    global fdiaryh;%full diary output handle or storage cell
    global bdiaryh;%brief diary output handle or storage cell
    if iscell(fdiaryh), fdiaryh=cell(0);
    else bdiaryh=cell(0);
    end
    set(handles.diary_LB, 'String', cell(0), 'Value', 1);
    
% debug enable on/off
function debug_TB_Callback(hObject, eventdata, handles)
    if get(hObject, 'Value'),
        % turn debug on
        set(hObject, 'String', 'Turn Off');
        set(handles.debug_T,'String','Debug: ON');
        for i=1:length(handles.tasks)
            if isfield(handles.tasks{i}, 'argin') && ...
                    isfield(handles.tasks{i}.argin,'deb') && ...
                    ~handles.tasks{i}.argin.deb,
                handles.tasks{i}.argin.deb=true;
            end
        end
    else
        % turn debug off
        set(hObject, 'String', 'Turn On');
        set(handles.debug_T,'String','Debug: OFF');
        for i=1:length(handles.tasks)
            if isfield(handles.tasks{i}, 'argin') && ...
                    isfield(handles.tasks{i}.argin,'deb') && ...
                    handles.tasks{i}.argin.deb,
                handles.tasks{i}.argin.deb=false;
            end
        end
    end
    handles=refresh_tasklist(handles, get(handles.task_list_LB,'Value'));
    % Update handles structure
    guidata(hObject, handles);
    
% clear debug ( matlab window )
function clear_debug_PB_Callback(hObject, eventdata, handles)
    clc;

%--------------------------------------------------------------------------
% Graphics
%--------------------------------------------------------------------------
% graphics output enable on/off callback
function graphicsout_TB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    if get(hObject, 'Value'),
        set(hObject, 'String', 'Turn Off');
        set(handles.graphics_T,'String','Graphics: ON');
    else
        set(hObject, 'String', 'Turn On');
        set(handles.graphics_T,'String','Graphics: OFF');
    end

% selected axes textfield callback
function selaxes_ET_Callback(hObject, eventdata, handles)
    graphics_out(handles,handles.P.xbox);

% repaint the graphics callback
function redraw_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    graphics_out(handles,handles.P.xbox);

% clear the graphics, remove the multiplot and restore the singe axes
function clearplot
    global gloutput;    % global output structure
    if isfield(gloutput, 'graphics')
        if isfield(gloutput.graphics,'ahand') && all(ishandle(gloutput.graphics.ahand))
            delete(gloutput.graphics.ahand);
        end
        gloutput=rmfield(gloutput, 'graphics');
    end

% plot the constraints and the xbox (in not empty)
function graphics_out(handles,xbox)
    global gloutput;    % global output structure
    global glconf;      % global configuration file
    if get(handles.graphicsout_TB, 'Value')
        nx=length(xbox.inf);
        if isfield(glconf,'hdmaxvars'), maxv=glconf.hdmaxvars;
        else maxv=5;
        end
        if nx>2
            sa=get(handles.selaxes_ET, 'String');
            if strcmp(sa,'original') || strcmp(sa,'orig')...
                    || strcmp(sa,'original vars')
                xind=1:min(handles.P.Q.nxo,maxv);
            else xind=str2num(sa);
            end  
            if ~isempty(xind) && all(xind<=nx) && all(xind>=0) &&...
                    length(xind)<=maxv
                glconf.xind=xind;
                if isfield(gloutput, 'graphics')
                    gloutput.graphics.xind=xind;
                end
                set(handles.selaxes_ET, 'Foreground','k');
            else
                set(handles.selaxes_ET, 'Foreground','r');
                return;
            end
        end
        if isfield(gloutput, 'graphics') && nx==2
            % clear the plot
            axes(gloutput.graphics.ahand(1));cla;
        end
        glplot(handles.P, xbox);
        drawnow;
    end
    
% set the default axe ranges callback
function default_axes_PB_Callback(hObject, eventdata, handles)
    set_default_axes(handles)

% set the default graphics axe ranges (from the configuration)
function set_default_axes(handles)
    global glconf;      % global configuration structure
    global gloutput;    % global output structure
    % get the reference box
    if isfield(gloutput,'graphics') 
        nx=length(gloutput.graphics.refbox.inf);
        if isfield(glconf, 'refboxran')
            gloutput.graphics.refbox.inf=zeros(nx,1)+glconf.refboxran(1);
            gloutput.graphics.refbox.sup=zeros(nx,1)+glconf.refboxran(2);
        else
            gloutput.graphics.refbox.inf=zeros(nx,1)-10;
            gloutput.graphics.refbox.sup=zeros(nx,1)+10;
        end
        setrange(gloutput.graphics.refbox);
    end
    
% zoom the graphics to the current box callback.
function zoom_TB_Callback(hObject, eventdata, handles)
    global glconf;         % global configuration variable
    global gloutput;       % global output structure
    INF=1e5;               % positive and negative infinity
    EXTP=0.05;             % extra area percent
    BPREC=4;               % precision of the box output
    RAN=[-15 15];          % default axe ranges
    
    % set constants from config
    if isfield(glconf,'infty'),    INF=glconf.infty;end     
    if isfield(glconf,'zoombound'),EXTP=glconf.zoombound;end
    if isfield(glconf,'boxprec'),  BPREC=glconf.boxprec;end
    if isfield(glconf,'refboxran'),RAN=glconf.refboxran;end
    
    % set too small box for graphics
    ZTHIN=10^(-BPREC+1);
    
    if isfield(handles,'P') && isfield(handles.P,'xbox') &&...
            isfield(gloutput,'graphics')
        zoomed=get(handles.zoom_TB, 'Value');
        if zoomed, set(handles.zoom_TB, 'String', 'Zoom Out');
        else set(handles.zoom_TB, 'String', 'Zoom In');
        end
        if zoomed
            % zoom out
            xbox=handles.P.xbox;
            if ~isempty(xbox)
                nx=length(xbox.inf);wid=xbox.sup-xbox.inf;
                box.inf=zeros(nx,1);box.sup=zeros(nx,1);
                for i=1:nx
                    if wid(i)>INF
                        % infinite box component
                        box.inf(i)=RAN(1);box.sup(i)=RAN(2);
                        if xbox.inf(i)>-INF, 
                            box.inf(i)=xbox.inf(i)-10*EXTP;
                        elseif xbox.sup(i)<INF,
                            box.sup(i)=xbox.sup(i)+10*EXTP;
                        end
                    else
                        % finite box component
                        ex=wid(i)*EXTP;
                        % very thin box
                        if ex<ZTHIN, ex=ZTHIN;end;
                        % set box component
                        box.inf(i)=xbox.inf(i)-ex;
                        box.sup(i)=xbox.sup(i)+ex;
                    end
                end
                if any(box.inf~=gloutput.graphics.refbox.inf) || ...
                        any(box.sup~=gloutput.graphics.refbox.sup)
                    % zoom to xbox
                    setrange(box);
                    handles.oldrefbox=gloutput.graphics.refbox;
                    gloutput.graphics.refbox=box;
                    guidata(hObject, handles); 
                else
                    % axe ranges did not change, zoom out
                    set(handles.zoom_TB, 'Value', 0, 'String', 'Zoom In');
                end
            else
                % xbox empty, zoom out
                set(handles.zoom_TB, 'Value', 0, 'String', 'Zoom In');
            end
        elseif isfield(gloutput,'graphics') && ...
                isfield(handles, 'oldrefbox')
            if length(handles.oldrefbox.inf)==...
                    length(gloutput.graphics.refbox.inf)
                setrange(handles.oldrefbox);
                gloutput.graphics.refbox=handles.oldrefbox;
            end
            handles=rmfield(handles, 'oldrefbox');
            guidata(hObject, handles);
        end
    else
        % problem empty, zoom out
        set(handles.zoom_TB, 'Value', 0, 'String', 'Zoom In');
    end
    
function setrange(ran)
    global gloutput;       % global output structure
    I=gloutput.graphics.I;
    n2=size(I,2);
    for i=1:n2
        axes(gloutput.graphics.ahand(i));
        axis([ran.inf(I(1,i)) ran.sup(I(1,i)) ran.inf(I(2,i))...
            ran.sup(I(2,i))]);
    end
%--------------------------------------------------------------------------
% Xbox, boxes and stack
%--------------------------------------------------------------------------
% set the xbox in the xbox LB
function set_xbox(handles)
    set(handles.set_xbox_PB, 'Enable', 'on');
    set(handles.clear_xbox_PB, 'Enable', 'on');
    %set xbox
    if ~isempty(handles.P.xbox),
        xbox=[handles.P.xbox.inf handles.P.xbox.sup];
    else
        nx=length(handles.P.Q.int);xbox=nan+zeros(nx,2);
    end
    %check boxes & stack
    if (isfield(handles.P,'boxes') && ~isempty(handles.P.boxes)) || ...
        (isfield(handles.P,'bstack') && ~isempty(handles.P.bstack))
        set(handles.showboxes_PB, 'Enable', 'on');
        set(handles.clearboxes_PB, 'Enable', 'on');
    else
        set(handles.showboxes_PB, 'Enable', 'off');
        set(handles.clearboxes_PB, 'Enable', 'off');
    end
    set(handles.xbox_LB, 'String', glmat2scell(xbox),...
        'Value', size(xbox,1));
        
% set xbox callback
function set_xbox_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    rans=get(handles.xbox_LB, 'String');
    var=get(handles.xbox_LB, 'Value');
    ran=str2num(rans{var});
    ran=glboxset_gui(var, ran);
    if ~isempty(ran)
        handles.P.xbox.inf(var)=ran(1);
        handles.P.xbox.sup(var)=ran(2);
        %set xbox
        set_xbox(handles);
        graphics_out(handles,handles.P.xbox);
        % Update handles structure
        guidata(hObject, handles);
    end
    
% clear xbox callback
function clear_xbox_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    nx=length(handles.P.Q.int);
    handles.P.xbox.inf = zeros(nx,1)-inf;
    handles.P.xbox.sup = zeros(nx,1)+inf;
    %set xbox
    set_xbox(handles);
    graphics_out(handles,handles.P.xbox);
    % Update handles structure
    guidata(hObject, handles);
    
% show all boxes obtained from a split or a cluster.
function showboxes_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    if ~isfield(handles.P,'bstack') && ~isfield(handles.P,'boxes')
        return;
    end
    global glconf;      % gloptlab configuration file
    global gloutput;    % global output structure
    NCOL=[0.9 0.9 0.9]; % color of the not processed boxes (pause)
    FCOL=[1 0.8 1];     % color of the found box
    FFCOL=[1 0 0];      % color of the found full box
    % set configuration
    if isfield(glconf,'nboxcol'),NCOL=glconf.nboxcol;end
    if isfield(glconf,'fboxcol'),FCOL=glconf.fboxcol;end
    if isfield(glconf,'ffboxcol'),FFCOL=glconf.ffboxcol;end
    gra=get(handles.graphicsout_TB, 'Value');
    if gra,
        %show boxes on the plot
        ghand=[];
        if isfield(gloutput,'graphics')
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=1;
        end
        if isfield(handles.P, 'bstack')
            %draw the boxes on stack
            for i=1:length(handles.P.bstack)
                h=glboxplot(handles.P.bstack{i}, 'Color', NCOL, 'Fill',...
                    inf, 'Refbox', handles.P.xbox);
                if ishandle(h),ghand=[ghand h];end
            end
        end
        if isfield(handles.P, 'boxes')
            for i=1:length(handles.P.boxes)
                % plot found box
                if handles.P.boxes{i}.status=='f';
                    h=glboxplot(handles.P.boxes{i}, 'Color', FFCOL, ...
                        'Fill', inf, 'Refbox', handles.P.xbox);
                else
                    h=glboxplot(handles.P.boxes{i}, 'Color', FCOL, ...
                        'Fill', inf, 'Refbox', handles.P.xbox);
                end
                if ishandle(h),ghand=[ghand h];end
            end
        end
        nhn=length(ghand);
        if nhn,
            hc=get(gca,'Children');
            set(gca,'Children',hc([nhn+1:end,1:nhn]));
        end
        drawnow;
    end
    %show the boxes as a text
    if isfield(handles.P, 'bstack') && isfield(handles.P, 'boxes')
        glboxesinfo_gui('Boxes', handles.P.boxes, 'Stack', ...
            handles.P.bstack);
    elseif isfield(handles.P, 'boxes'),
        glboxesinfo_gui('Boxes', handles.P.boxes);
    else
        glboxesinfo_gui('Stack', handles.P.bstack.data);
    end
    if gra,
        if nhn,
            delete(ghand);
        else
            gloutput.graphics.imdat=zeros(size(gloutput.graphics.imdat));
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=1;
            glboxplot(gloutput.graphics.refbox);
        end
    end
    
% clear all the found boxes and the stack callback
function clearboxes_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    handles.P=rmfield(handles.P,'bstack');
    handles.P=rmfield(handles.P,'boxes');
    set(handles.showboxes_PB, 'Enable', 'off');
    set(handles.clearboxes_PB, 'Enable', 'off');
    % Update handles structure
    guidata(hObject, handles);

%--------------------------------------------------------------------------
% Directory manipulation
%--------------------------------------------------------------------------
% get current directory callback 
function current_dir_ET_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    d=get(handles.current_dir_ET, 'String');
    if exist(d,'dir') == 7
        cd(d);handles.sdir='';
        handles=get_directory(handles);
        % Update handles structure
        guidata(hObject, handles);
    else
        h=warndlg(['Directory ' d ' does not exists!'],'modal');
        uiwait(h)
    end
    
% set files and directories
function dirs_LB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    dirs = get(hObject,'String');
    d=dirs{get(hObject,'Value')};
    if strcmp(d,handles.sdir);
        cd(d);handles.sdir='';
        handles=get_directory(handles);
        %refresh problist
        refresh_plist(handles);
    else
        handles.sdir=d;
    end
    button_check(handles);
    % Update handles structure
    guidata(hObject, handles);

% refesh the problem list, remove all non random problems
function refresh_plist(handles)
    if break_strategy(handles), return; end
    i=1;probs=get(handles.prob_list_LB, 'String');
    while i <= length(probs)
        if ~isempty(findstr(probs{i}, 'rand')) ...
                && isempty(findstr(probs{i},'.glb'))
            i=i+1;
        else
            probs(i)=[];
        end
    end
    set(handles.prob_list_LB, 'Value', 1, 'String', probs);
    
% set the sort type callback
function sorttype_TB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    state=get(hObject, 'Value');
    if state,
        set(hObject, 'String', 'Sizesort');
        handles=get_directory(handles);
    else
        set(hObject, 'String', 'Namesort');
        handles=get_directory(handles);
    end
     % Update handles structure
    guidata(hObject, handles);
    
% get the directory and files listing
function handles=get_directory(handles)
    %get dirs
    set(handles.current_dir_ET, 'String', cd);
    direct=dir(cd);
    dirs={direct.name};isd=[direct.isdir];
    dirs=dirs(isd==1);
    set(handles.dirs_LB, 'Value', 1, 'String', dirs);
    %file extension check and get file listing
    ext=get(handles.file_ext_ET,'String');
    ind=[0 find(ext==',') length(ext)+1];files=[];
    for i=1:length(ind)-1
        ex=ext(ind(i)+1:ind(i+1)-1);
        ex=ex(ex~=' ');%remove spaces 
        if ~isempty(strfind(ex,'.*'))
            ex=ex(1:find(ex=='.')-1);
            files=[files ; dir([ex '.def'])];
            files=[files ; dir([ex '.glb'])];
            files=[files ; dir([ex '.mod'])];
            files=[files ; dir([ex '.dag'])];
            files=[files ; dir([ex '.sml'])];
        elseif  ~isempty(strfind(ex,'.def')) ||...
                ~isempty(strfind(ex,'.glb')) ||...
                ~isempty(strfind(ex,'.mod')) ||...
                ~isempty(strfind(ex,'.dag')) ||...
                ~isempty(strfind(ex,'.sml'))
            files=[files ; dir(ex)];
        else
            h=warndlg([{['Wrong problem file filter: ' ex ' !']} ; ...
                {['( Select one or more of the extensions: .*,'...
                 ' .def, .glb .mod .sml or .dag)']}]...
                ,'modal');
            uiwait(h)
            set(handles.files_LB, 'Value', 1, 'String', []);
            return; 
        end
    end
    %sort files
    if get(handles.sorttype_TB, 'Value')
        %sizesort
        [s,ind]=sort([files.bytes]);
        files={files(ind).name};
    else
        %namesort
        files={files.name};
        files=sort(files);
    end
    %remove duplicate entries
    i=1;l=length(files);
    while i<l
        j=i+1;
        while j<l
            if strcmp(files{i},files{j}),files(j)=[];l=l-1;
            else break;
            end
        end
        i=i+1;
    end
    set(handles.files_LB, 'Value', 1, 'String', files);
    %deactivate buttons
    if ~isempty(files)
        set(handles.add_prob_PB, 'Enable', 'on');
        set(handles.add_all_probs_PB, 'Enable', 'on');
    else 
        set(handles.add_prob_PB, 'Enable', 'off');
        set(handles.add_all_probs_PB, 'Enable', 'off');
    end
    set(handles.set_xbox_PB, 'Enable', 'off');
    set(handles.clear_xbox_PB, 'Enable', 'off');
    set(handles.edit_file_PB, 'Enable', 'off');
    set(handles.convert_PB, 'Enable', 'off');
    button_check(handles);
    
%--------------------------------------------------------------------------
% File manipulation
%--------------------------------------------------------------------------
% set file extension callback
function file_ext_ET_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    handles=get_directory(handles);
    % Update handles structure
    guidata(hObject, handles);

% refesh file list callback
function refresh_files_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    handles=get_directory(handles);
    % Update handles structure
    guidata(hObject, handles);

% delete the selected file callback
function delete_file_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    fnames = get(handles.files_LB,'String');
    if (isempty(fnames)), return; end
    fname=fnames{get(handles.files_LB,'Value')};
    h = questdlg(['Delete the problem file: '...
        fname '?' ],'Delete File','Yes','No','No');
    if strcmp(h,'Yes')
        %delete file
        delete(fname);
        %remove it from the problem list(if present)
        probs=get(handles.prob_list_LB,'String');
        if ~isempty(probs)
            %find all indexes & remove them from prob list
            ind=glfindstr(probs,fname);
            probs(ind)=[];
            %set new problem listing
            ind=max(1,min(get(handles.prob_list_LB,'Value'), length(probs)));
            set(handles.prob_list_LB,'Value',ind,'String',probs);
        end
        %save current index in the file window (max new length)
        ind=min(get(handles.files_LB,'Value'), length(fnames)-1);
        %get new directory listing
        handles=get_directory(handles);
        %set index in file window
        set(handles.files_LB,'Value',ind(1));
        fnames = get(handles.files_LB,'String');
        %set problem info
        probs_files_sync(handles,0);  
        handles=clear_problem(handles);
        if ~isempty(fnames)
            set_problem_info(handles, fnames{ind});
        end
        % Update handles structure
        guidata(hObject, handles);
    end

% file selection callback
function files_LB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    handles=set_file(handles);
    % Update handles structure
    guidata(hObject, handles);
    
% set the diffrent conversion buttons according to the file type active or
% inactive, synchronize the selected problmes to the selected file and
% clear the problem loaded.
function handles=set_file(handles)
    global glconfstat;
    fnames = get(handles.files_LB,'String');
    if isempty(fnames)
        return
    end
    fname=fnames{get(handles.files_LB,'Value')};
    %problem  file clicked, set problem information
    set_problem_info(handles, fname);
    %deactivate description save button
    set(handles.probdesc_ET, 'Enable', 'inactive');
    set(handles.save_pinfo_PB, 'Enable', 'off');
    %activate buttons
    set(handles.add_prob_PB, 'Enable', 'on');
    if ~isempty(strfind(fname, '.glb')) || ...
            ~isempty(strfind(fname, '.def')) ||...
            ~isempty(strfind(fname, '.sml')) ||...
            ~isempty(strfind(fname, '.mod')) && isunix ||...
            ~isempty(strfind(fname, '.dag')) && isunix
        set(handles.convert_PB, 'Enable', 'on');
    else
        set(handles.convert_PB, 'Enable', 'off');
    end

    if ~isempty(strfind(fname, '.mod')) || ~isempty(strfind(fname, '.sml'))
        set(handles.edit_file_PB, 'Enable', 'on');
    else
        set(handles.edit_file_PB, 'Enable', 'off');
    end

    if ~isempty(strfind(fname, '.dag')) && glconfstat.coconut_ok
        set(handles.draw_dag_PB, 'Enable', 'on');
    else
        set(handles.draw_dag_PB, 'Enable', 'off');
    end
    
    if ~isempty(strfind(fname, '.dag'))
        set(handles.view_dag_PB, 'Enable', 'on');
    else
        set(handles.view_dag_PB, 'Enable', 'off');
    end
    probs_files_sync(handles,1);   
    handles=clear_problem(handles);
    

% read a problem, .def or .glb, box=1 means the eventually saved box will 
% be also read from a glb file
function handles=read_problem(handles, fname, prt, deb, box)

    if ~isempty(findstr(fname, 'rand')) && isempty(find(fname=='.', 1))
        %random problem 
        for j=1:length(handles.random)
            if strcmp(handles.random{j}.name, fname)
                handles.P=handles.random{j};readerr=-1;
                break;
            end
        end
        %set xbox
        handles.P.xbox.inf=-inf+zeros(length(handles.P.Q.int),1);
        handles.P.xbox.sup=inf+zeros(length(handles.P.Q.int),1);
    else
        [handles.P, readerr]=glread(fname, box);
        if readerr==1,
            diary_full_out(['Problem file ' fname ' could not be read!']);
            handles.P=[];return;
        elseif readerr==2,
            diary_full_out('File type not supported, use a converter!');
            handles.P=[];return;
        elseif readerr==3,
            diary_full_out(['Problem file ' fname ' is missing!']);
            handles.P=[];return;
        end
    end
    set_xbox(handles);
    %text output
    if prt,
        diary_full_out(char(zeros(1,75)+45));
        if readerr==0, diary_full_out(['Problem file ' fname ' read!']);
        else diary_full_out(['Random problem ' fname ' initialized!']);
        end
        diary_full_out(char(zeros(1,75)+45));
        diary_full_out(glprob2str(handles.P));
    end
    
%--------------------------------------------------------------------------
% Problem information
%--------------------------------------------------------------------------
% set problem information for prob = filename or prob = P
function set_problem_info(handles, prob)
    set(handles.probname_ET, 'String', []);
    set(handles.probdesc_ET, 'String', []);
    set(handles.nx_T, 'String', []);
    set(handles.no_T, 'String', []);
    set(handles.ni_T, 'String', []);
    set(handles.nc_T, 'String', []);
    set(handles.nB_T, 'String', []);
    set(handles.ne_T, 'String', []);
    set(handles.nu_T, 'String', []);
    set(handles.nA_T, 'String', []);
    set(handles.probdesc_ET, 'Enable', 'inactive');
    set(handles.save_pinfo_PB, 'Enable', 'off');
    if isstruct(prob) || ~isempty(findstr(prob,'.glb')) ||...
            ~isempty(findstr(prob,'.def'))
        [info, errmsg] = glreadinfo(prob);
        if isempty(errmsg)
            set(handles.probname_ET, 'String', info.name);
            set(handles.probdesc_ET, 'String', info.desc);
            set(handles.nx_T, 'String', num2str(info.nx));
            set(handles.no_T, 'String', num2str(info.no));
            set(handles.ni_T, 'String', num2str(info.ni));
            set(handles.nc_T, 'String', num2str(info.nc));
            set(handles.nB_T, 'String', num2str(info.nB));
            set(handles.ne_T, 'String', num2str(info.ne));
            set(handles.nu_T, 'String', num2str(info.nu));
            set(handles.nA_T, 'String', num2str(info.nA));
            if ischar(prob) && ~isempty(findstr(prob,'.glb'))
                set(handles.probdesc_ET, 'Enable', 'on');
                set(handles.save_pinfo_PB, 'Enable', 'on');
            end
        else
            diary_full_out(errmsg);
            diary_full_out('Try to refresh the directory!');
            %get new directory listing
            %handles=get_directory(handles);
            %refresh_plist(handles);
        end
    end
    
    
% save the edited problem infromation callback
function save_pinfo_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    probs=get(handles.prob_list_LB,'String');
    fname=probs{get(handles.prob_list_LB,'Value')};
    fname=[fname(1:find(fname=='.')-1) '.inf'];
    info.name=get(handles.probname_ET, 'String');
    info.desc=get(handles.probdesc_ET, 'String');
    info.nx=num2str(get(handles.nx_T, 'String'));
    info.no=num2str(get(handles.no_T, 'String'));
    info.ni=num2str(get(handles.ni_T, 'String'));
    info.nc=num2str(get(handles.nc_T, 'String'));
    info.nB=num2str(get(handles.nB_T, 'String'));
    info.ne=num2str(get(handles.ne_T, 'String'));
    info.nu=num2str(get(handles.nu_T, 'String'));
    info.nA=num2str(get(handles.nA_T, 'String'));
    glsave(info, fname);

%--------------------------------------------------------------------------
% Problem list manipulation
%--------------------------------------------------------------------------
% add the selected problem to the problem list callback
function add_prob_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    fnames = get(handles.files_LB,'String');
    fname=fnames{get(handles.files_LB,'Value')};
    [d, name, ext] = fileparts(fname);
    if strcmp(ext,'.def') || strcmp(ext,'.glb')
        handles=add_prob(handles,fname);
        set(handles.remove_prob_PB, 'Enable', 'on');
        probs_files_sync(handles,1);
        handles=clear_problem(handles);
    elseif strcmp(ext,'.sml')
        status = glsystem(['sml2glb ' fname]);
        if ~status,
            fname=[name '.glb'];
            handles=get_directory(handles);
            files=get(handles.files_LB,'String');
            ind=glfindstr(files, fname);
            if ~isempty(ind)
                set(handles.files_LB, 'Value',ind(1));
                handles=set_file(handles);
                handles=add_prob(handles,fname);
                set(handles.remove_prob_PB, 'Enable', 'on');
                probs_files_sync(handles,1);
                handles=clear_problem(handles);
            end
            glsound('tfinish');
        else
            diary_full_out(['Cannot convert the file, check the content '...
                'or try to use the convert option!']);
            glsound('error');
        end
    end
    % Update handles structure
    guidata(hObject, handles);
    
% add a problem name to the problist if it is not already there
function handles=add_prob(handles,pname)
    probs=get(handles.prob_list_LB,'String');
    ind=glfindstr(probs, pname);
    if isempty(ind)
        problist=[get(handles.prob_list_LB,'String') ; {pname}];
        set(handles.prob_list_LB,'String', problist);
    end
      
% add all problems in the active directory to the problem list, callback
function add_all_probs_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    fnames=get(handles.files_LB, 'String');
    i=1;n=length(fnames);
    %remove non supported file types
    while i<=n
        if isempty(strfind(fnames{i},'.def')) &&...
                isempty(strfind(fnames{i},'.glb'))
            fnames(i)=[];n=n-1;
        else i=i+1;
        end
    end    
    for i=1:length(fnames)
        handles=add_prob(handles,fnames{i});
    end
    set(handles.remove_prob_PB, 'Enable', 'on');
    probs_files_sync(handles,0);    
    handles=clear_problem(handles);
    % Update handles structure
    guidata(hObject, handles); 


% generate some random problems and add it to the problem list, callback
function random_prob_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    %make cell of random probs
    P=glrandomprob_gui;
    %find all in this directory  existing random problemfile numbers
    %together with the already created radom problems
    names = [get(handles.files_LB,'String');get(handles.prob_list_LB,'String')];
    nums=[];ind = glfindstr(names, 'rand');
    for i=ind
        name=names{i};num=str2num(name(5:7));
        if ~isempty(num), nums=[nums num];end
    end
    %name random probs
    probs=get(handles.prob_list_LB,'String');
    rcount=1;
    for i=1:length(P)
        while any(nums==rcount)
            rcount=rcount+1;
        end
        num=num2str(rcount);
        P{i}.name=['rand' char(zeros(1,3-length(num))+48) num];
        rcount=rcount+1;
        handles.random=[handles.random; {P{i}}];
        probs=[probs ; {P{i}.name}];
        set(handles.prob_list_LB, 'String', probs);
        set(handles.remove_prob_PB, 'Enable', 'on');
        %set information
        set_problem_info(handles, P{i})
    end
    handles=clear_problem(handles);
    % Update handles structure
    guidata(hObject, handles); 
  

% save the highlied problem to a .glb file callback
function save_prob_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    probs=get(handles.prob_list_LB,'String');
    if isempty(probs), return; end
    pind=get(handles.prob_list_LB,'Value');fname=probs{pind};
    if length(handles.tasks)>2
        strategy.name=get(handles.strategy_file_ET, 'String');
        strategy.tasks=handles.tasks;
    else
        strategy=[];
    end
    if isfield(handles, 'P') && ~isempty(handles.P) && ...
        ~isempty(findstr(fname,handles.P.name))
        %problem file already loaded
        %transform equality constraints back to inequs when they were
        %transformed
        P=handles.P;
        if isfield(handles.P.Q,'slacks')
            P=handles.P;
            P.xbox.inf=P.xbox.inf(1:nx);
            P.xbox.sup=P.xbox.sup(1:nx);
            P.Q.int=P.Q.int(1:nx);
            ind=1:nx;ind2=ind;
            for i=1:nx,ind2=[ind2 i*(nx+ns)+ind];end
            P.Q.A.inf=P.Q.A.inf(:,ind2);
            P.Q.A.sup=P.Q.A.sup(:,ind2);
            P.Q=glfindspars(P.Q);
            %disp(glprob2str(P));
        end
        fname=glsaveprob_gui('Problem',P,'Strategy',strategy);
    elseif isempty(findstr(fname,'.glb')) && isempty(findstr(fname,'.def'))
        %random file not loaded
        ind2=glfindstr(handles.random, fname, 'name');
        fname=glsaveprob_gui('Problem', handles.random{ind2}, 'Strategy',...
            strategy);
        if ~isempty(fname),
            % remove saved random problem, from random list
            handles.random(ind2)=[];
        end
    else
        %.glb or .def file not loaded
        box=handles.tasks{1}.argin.box;
        handles=read_problem(handles, fname, 0, 0, box);
        fname=glsaveprob_gui('Problem',handles.P,'Strategy',strategy);
    end
    if ~isempty(fname)
        %set new filename in problem list
        probs{pind}=[fname '.glb'];
        set(handles.prob_list_LB, 'String', probs);
        handles=get_directory(handles);
        probs_files_sync(handles,0);
        handles=clear_problem(handles);
        % Update handles structure
        guidata(hObject, handles);
    end

% removes the highlied problem
function remove_prob_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    probs=get(handles.prob_list_LB, 'String');
    ind=get(handles.prob_list_LB, 'Value');len=length(probs);
    if len>1
        if ~isempty(findstr(probs{ind}, 'rand')) ...
            && isempty(findstr(probs{ind},'.glb'))
            %remove random problem
            for i=1:length(handles.random)
                if strcmp(handles.random{i}.name, probs(ind))
                    if length(handles.random)>1
                        handles.random=[handles.random(1:i-1);handles.random(i+1:end)];
                    else
                        handles.random=cell(0);
                    end
                    break;
                end
            end
        end
        %remove entry from problist
        probs=[probs(1:ind-1);probs(ind+1:end)];
        ind=min(ind,length(probs));
    else
        probs=cell(0);ind=1;
        handles.random=cell(0);
        set(handles.remove_prob_PB, 'Enable', 'off');
    end
    set(handles.prob_list_LB, 'Value', ind, 'String', probs);
    probs_files_sync(handles,0);
    handles=clear_problem(handles);
    % Update handles structure
    guidata(hObject, handles);
    
 % sort the problem list and remove duplicately added problems, callback
function sort_problist_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    probs=get(handles.prob_list_LB, 'String');
    probs=sort(probs);n=length(probs);
    %remove duplicate entries
    i=1;
    while i<n
        j=i+1;
        while j<=n
            if strcmp(probs{i},probs{j}), probs(j)=[];n=n-1;
            else j=j+1;
            end
        end
        i=i+1;
    end
    set(handles.prob_list_LB, 'Value', 1, 'String', probs);
    probs_files_sync(handles,0);
    handles=clear_problem(handles);
    % Update handles structure
    guidata(hObject, handles);
    
% remove all problems from the problem list, callback
function clear_prob_list_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    handles.random=cell(0);
    set(handles.prob_list_LB, 'Value', 1, 'String', cell(0));
    set(handles.remove_prob_PB, 'Enable', 'off');
    handles=clear_problem(handles);
    % Update handles structure
    guidata(hObject, handles);

% set problem info and and syncs when the problem list is used, callback
function prob_list_LB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    probs=get(handles.prob_list_LB, 'String');
    if ~isempty(probs)
        prob=probs{get(handles.prob_list_LB, 'Value')};
        if ~isempty(findstr(prob, 'rand')) && isempty(find(prob=='.', 1))
            for i=1:length(handles.random)
                if strcmp(handles.random{i}.name, prob)
                    %set problem info
                    set_problem_info(handles, handles.random{i});
                    break;
                end
            end
        else
           %set problem info
           set_problem_info(handles, prob);
        end
       probs_files_sync(handles,0);
       handles=clear_problem(handles);
       % Update handles structure
       guidata(hObject, handles);
    end

   
% syncronize the files and the problems windows by setting
% the focus on the same file if its in both of the windows.
% If flag=1 it syncronizes probs to the selected file else the other way.
function probs_files_sync(handles,flag)
    files=get(handles.files_LB,'String');
    probs=get(handles.prob_list_LB, 'String');
    if ~isempty(files) && ~isempty(probs),
        if flag
            ind=glfindstr(probs,files{get(handles.files_LB,'Value')});
            if ~isempty(ind)
                set(handles.prob_list_LB, 'Value',ind(1));
            end
        else
            ind=glfindstr(files,probs{get(handles.prob_list_LB, 'Value')});
            if ~isempty(ind)
                set(handles.files_LB, 'Value',ind(1));
            end
        end
    end

% clear the handles.P, the plot, xbox , set the default axes and reset 
% the strategy
function handles=clear_problem(handles)
    handles.P=[];
    clearplot;set_default_axes(handles);
    set(handles.xbox_LB,'String',cell(0));
    set(handles.task_list_LB, 'Value', 1);
    set(handles.redraw_PB, 'Enable', 'off');
    set(handles.zoom_TB, 'Enable', 'off');
    button_check(handles);
    
%--------------------------------------------------------------------------
% Strategy manipulation
%--------------------------------------------------------------------------
% strategy manipulation callback
function task_list_LB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    button_check(handles);
    % Update handles structure
    guidata(hObject, handles);

% set strategy output type
function strategy_full_out_CB_Callback(hObject, eventdata, handles)
    handles=refresh_tasklist(handles);
    % Update handles structure
    guidata(hObject, handles);

% check the syntax of the strategy contained in the task_list_LB
function check_strategy(handles)
    [status, msg] = glchktskl(handles.tasks);
    if ~isempty(msg)
        diary_full_out(msg);
        diary_brief_out(msg);
    end
    button_check(handles);
    
% refresh task_list_LB callback
function handles=refresh_tasklist(handles, ind)
    if nargin<2, ind=get(handles.task_list_LB,'Value'); end
    len=length(handles.tasks);
    taskn=cell(len,1);
    spac=' ';
    for i=1:len
        % spacing
        if ~isempty(strfind(handles.tasks{i}.name,'End ')) && length(spac)>1
            %End Split & End While
            spac=spac(1:end-7);
        end
        %line numeration format
        if i<10, num='0';else num=[];end
        %create text
        def=~get(handles.strategy_full_out_CB, 'Value');
        str=gltask2str(handles.tasks{i}, def);
        if isfield(handles.tasks{i},'text'),
            error('refresh_tasklist: wrong task!');
        end
        if handles.tasks{i}.comm==1
            %commented task
            taskn{i}=[num num2str(i) ':%' spac(2:end) str];
        else
            taskn{i}=[num num2str(i) ':' spac str];
        end
        % spacing 2
        if ~isempty(strfind(handles.tasks{i}.name,'Begin '))
            %Begin Split & Begin While
            spac=[spac '       '];
        end
    end
    set(handles.task_list_LB,'String', taskn, 'Value', ind);
    
    
% add a new task to the strategy callback
function new_task_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    %create task
    task=glselecttask_gui;
    if ~isempty(task),
        %put task in tasklist
        ind=get(handles.task_list_LB,'Value');
        if ~iscell(task), task={task};end;
        %set comment
        for i=1:length(task), task{i}.comm=false; end
        %set tasklist
        handles.tasks=[handles.tasks(1:ind-1) ;...
            task ; handles.tasks(ind:end)];
        set(handles.task_list_LB,'Value', ind+1);
        handles=refresh_tasklist(handles, ind+1);
        check_strategy(handles);
        % Update handles structure
        guidata(hObject, handles);
    end
        
% remove a task from the strategy callback
function remove_task_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    ind=get(handles.task_list_LB, 'Value');
    handles=remove_task(handles, ind);
    check_strategy(handles);
    % Update handles structure
    guidata(hObject, handles);
    
% remove the task with the given index from the strategy
function handles=remove_task(handles, ind)
    if ~isempty(strfind(handles.tasks{ind}.name, 'Begin '))
        %Split and While, find type and search for 'End (type)'
        tnam=handles.tasks{ind}.name;type=tnam(find(tnam==' ')+1:end);
        op=1;
        for k=ind+1:length(handles.tasks);
            if strcmp(handles.tasks{k}.name,['Begin ' type]),op=op+1;
            elseif strcmp(handles.tasks{k}.name,['End ' type]),op=op-1;
            end
            if op==0, ind=[ind k]; break; end
        end
    end
    handles.tasks(ind)=[];
    ind=min(ind(1),length(handles.tasks));
    handles=refresh_tasklist(handles, ind);
 
% modify a task in the strategy callback
function modify_task_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    %get active task
    ind=get(handles.task_list_LB,'Value');
    task=handles.tasks{ind};
    task=glselecttask_gui('Task', task);
    if ~isempty(task)
        %replace task
        if iscell(task), task=task{1};end;
        %set comment
        task.comm=false;
        %set tasklist
        handles.tasks{ind}=task;
        handles=refresh_tasklist(handles, ind);
        check_strategy(handles);
        % Update handles structure
        guidata(hObject, handles);
    end
    
% replace a task in the strategy with another, callback
function replace_task_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    %create task
    task=glselecttask_gui;
    if ~isempty(task)
        %replace task
        ind=get(handles.task_list_LB,'Value');
        handles=remove_task(handles,ind);
        if ~iscell(task), task={task};end;
        %set comment
        for i=1:length(task), task{i}.comm=0; end
        %set tasklist
        handles.tasks=[handles.tasks(1:ind-1) ;...
                task ; handles.tasks(ind:end)];
        handles=refresh_tasklist(handles, ind);
        check_strategy(handles);
        % Update handles structure
        guidata(hObject, handles);
    end

% comment or uncomment a task callback
function comm_task_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    ind=get(handles.task_list_LB, 'Value');
    if handles.tasks{ind}.comm==0
        %comment task
        if ~isempty(strfind(handles.tasks{ind}.name, 'Begin '))
            %Split and While find type and search for 'End (type)'
            tnam=handles.tasks{ind}.name;type=tnam(find(tnam==' ')+1:end);
            op=0;
            for k=ind:length(handles.tasks)
                handles.tasks{k}.comm=true;
                if strcmp(handles.tasks{k}.name,['Begin ' type]),op=op+1;
                elseif strcmp(handles.tasks{k}.name,['End ' type]),op=op-1;
                end
                if op==0, break; end
            end
        else
            handles.tasks{ind}.comm=true;
        end
        set(hObject,'String', 'Uncomment');
    else
        if ~isempty(strfind(handles.tasks{ind}.name, 'Begin '))
            %Split and While find type and search for 'End (type)'
            tnam=handles.tasks{ind}.name;type=tnam(find(tnam==' ')+1:end);
            op=0;
            for k=ind:length(handles.tasks)
                handles.tasks{k}.comm=false;
                if strcmp(handles.tasks{k}.name,['Begin ' type]),op=op+1;
                elseif strcmp(handles.tasks{k}.name,['End ' type]),op=op-1;
                end
                if op==0, break; end
            end
        else
            handles.tasks{ind}.comm=false;
        end
        set(hObject,'String', 'Comment');
    end
    handles=refresh_tasklist(handles, ind);
    % Update handles structure
    guidata(hObject, handles);



% move the selected task upwards in the tasklist
function movetaskup_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    ind=get(handles.task_list_LB, 'Value');
    if ind>2
        % swap tasks
        task=handles.tasks{ind};
        handles.tasks{ind}=handles.tasks{ind-1};
        handles.tasks{ind-1}=task;
        check_strategy(handles);
        handles=refresh_tasklist(handles, ind-1);
        % Update handles structure
        guidata(hObject, handles);
    end



% move the selected task downwards in the tasklist
function movetaskdown_PB_Callback(hObject, eventdata, handles)
if break_strategy(handles), return; end
    ind=get(handles.task_list_LB, 'Value');
    if ind<length(handles.tasks)-1
        % swap tasks
        task=handles.tasks{ind};
        handles.tasks{ind}=handles.tasks{ind+1};
        handles.tasks{ind+1}=task;
        check_strategy(handles);
        handles=refresh_tasklist(handles, ind+1);
        % Update handles structure
        guidata(hObject, handles);
    end

% remove all tasks from the strategy callback
function clear_strategy_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    h = questdlg(['This action will clear the current strategy, are '...
        'you sure?'],'Clear the strategy','Yes','No','No');
    if strcmp(h,'Yes'),
        handles=clear_strategy(handles);
        handles=clear_problem(handles);
    end
    % Update handles structure
    guidata(hObject, handles);
    
% clears the whole strategy  
function handles=clear_strategy(handles)
    global glconf; %global configuration file
    % read problem task
    task1.name='Read Problem';
    task1.argin=glconf.read;
    task1.comm=false;
    % pause task
    task2.name='Pause';
    task2.comm=false;
    % end task
    task3.name='Finish';
    task3.comm=false;
    handles.tasks=[{task1};{task2};{task3}];
    handles=refresh_tasklist(handles, 2);
    button_check(handles);
    
% browse and load a saved strategy callback
function browse_strategy_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    global glconf;    %global configuration variable
    %select a strategy
    fname=glfileselect_gui('Path', glconf.stratdir, 'Chdir', 'inactive',...
        'Filemask','*.str', 'File', [get(handles.strategy_file_ET, ...
        'String') '.str']);
    if ~isempty(fname)
        %remove path and extension
        ind=find(fname==filesep);
        fname=fname(ind(end)+1:end-4);
        %load the strategy
        h = questdlg(['Loading the strategy ' fname '.str will replace '...
            'the current strategy, are you sure?'],'Load a strategy',...
            'Yes','No','No');
        if strcmp(h,'Yes'),
            handles=load_strategy(handles, fname);
        end
        % Update handles structure
        guidata(hObject, handles);
    end
    
% load strategy callback
function load_strategy_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    name=get(handles.strategy_file_ET, 'String');
    h = questdlg(['Loading the strategy ' name '.str will replace '...
        'the current strategy, are you sure?'],'Load a strategy',...
        'Yes','No','No');
    if strcmp(h,'Yes'),
        handles=load_strategy(handles, name);
    end
    % Update handles structure
    guidata(hObject, handles);

% load the strategy with the name given in the strategy_file textfield
function handles=load_strategy(handles, name)
    global glconf;
    sfile=[glconf.stratdir name '.str'];
    if exist(sfile,'file') == 2,
        % determine if a mat or a txt file
        fid=fopen(sfile,'r'); fline = fgetl(fid); fclose(fid);
        % load the strategy
        if fline(1)=='%', strategy=glload(sfile);%text file
        else load(sfile, '-mat');%mat-file
        end
        diary_full_out(['*Strategy file ' name '.str loaded!*']);
        if strcmp(strategy.tasks{end}.name,'End')
            % compatibility override for old strategies
            strategy.tasks{end}.name='Finish';
            handles.tasks=strategy.tasks;
            handles=refresh_tasklist(handles, 1);
            save_strategy(name, handles);
        else
            handles.tasks=strategy.tasks;
            handles=refresh_tasklist(handles, 1);
        end
        % check the strategy
        check_strategy(handles);
        % clear the problem
        handles=clear_problem(handles);
    else
        h=warndlg(['Strategy file ' name '.str not found, empty '...
            'strategy created!'],'File not found!','modal');
        uiwait(h)
        handles=clear_strategy(handles);
    end
    %set the strategy name
    set(handles.strategy_file_ET, 'String', name);

 function save_strategy(name, handles)
     global glconf;
     strategy.comment1='%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%';
     strategy.comment2='%%%%%%%%%%%%%%% GLOPTLAB STRATEGY FILE %%%%%%%%%%%%%%%';
     strategy.comment3='%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%';
     strategy.comment4='%';
     strategy.comment5=['% ' upper(name)];
     strategy.comment6='%-----------------------------------------------------';
     desc=get(handles.task_list_LB,'String');
     for i=1:length(desc)
         strategy.(['comment' num2str(i+6)])=['% ' desc{i}];
     end
     strategy.(['comment' num2str(length(desc)+7)])=...
         '%-----------------------------------------------------';
     strategy.(['comment' num2str(length(desc)+8)])='';
     strategy.name=name;strategy.name_comment='%strategy name';
     strategy.tasks=handles.tasks;strategy.tasks_comment='%tasklist';
     fname=[glconf.stratdir name '.str'];
     glsave(strategy, fname);
    
    
% saves the current strategy
function save_strategy_PB_Callback(hObject, eventdata, handles)
    global glconf;
    if break_strategy(handles), return; end
    name=get(handles.strategy_file_ET, 'String');
    fname=[glconf.stratdir name '.str'];
    if exist(fname,'file')==2,
        h = questdlg(['This action will overwrite the file ' name ...
        '.str, are you sure?'],'Save the strategy','Yes','No','No');
    else h='Yes';
    end
    if strcmp(h,'Yes'),
        save_strategy(name, handles);
        diary_full_out(['*Strategy file ' name '.str saved!*']);
    end
    

%--------------------------------------------------------------------------
% Processing the strategy
%--------------------------------------------------------------------------
% check if the strategy can be processed or not
function button_check(handles)
    % check task buttons
    ind=get(handles.task_list_LB,'Value');
    tname=handles.tasks{ind}.name;
    maxind=length(handles.tasks);
    if ind==1
        set(handles.new_task_PB, 'Enable', 'off');
        set(handles.remove_task_PB, 'Enable', 'off');
        set(handles.modify_task_PB, 'Enable', 'on');
        set(handles.replace_task_PB, 'Enable', 'off');
        set(handles.comm_task_PB, 'Enable', 'off');
        if isfield(handles,'P') && ~isempty(handles.P)
            handles=clear_problem(handles);
        end
    elseif ind==maxind || strcmp(tname,'End ')
        %Split and  While
        set(handles.new_task_PB, 'Enable', 'on');
        set(handles.remove_task_PB, 'Enable', 'off');
        set(handles.modify_task_PB, 'Enable', 'off');
        set(handles.replace_task_PB, 'Enable', 'off');
        set(handles.comm_task_PB, 'Enable', 'off');
    else
        set(handles.new_task_PB, 'Enable', 'on');
        set(handles.remove_task_PB, 'Enable', 'on');
        set(handles.modify_task_PB, 'Enable', 'on');
        set(handles.replace_task_PB, 'Enable', 'on');
        if handles.tasks{ind}.comm==1
            set(handles.comm_task_PB, 'Enable', 'on', 'Value', 1,...
                'String', 'Uncomment');
        else
            set(handles.comm_task_PB, 'Enable', 'on', 'Value', 0,...
                'String', 'Comment');
        end
    end
    if maxind==2
        set(handles.save_strategy_PB, 'Enable', 'off');
    else
        set(handles.save_strategy_PB, 'Enable', 'on')
    end
    % check tasklist syntax
    status = glchktskl(handles.tasks);
    % check execute button
    probs=get(handles.prob_list_LB, 'String');
    if ~isempty(probs) && status==0
        set(handles.execute_TB, 'Enable', 'on');
    else
        set(handles.execute_TB, 'Enable', 'off');
    end
    
% this function checks if the strategy currently being processed or not, if
% yes it interrupts the solution process, and returns 1, else returns 0. 
function br=break_strategy(handles)
    global glbreak;     %gloptlab break flag
    %computation check
    if get(handles.execute_TB, 'Value')==1
        %break the execution of the strategy
        glbreak=1;
        set(handles.execute_TB, 'Value', 0,'String','Execute');
        if get(handles.diary_TB, 'Value')
            diary_brief_out('*Unintentional user break!*');
            diary_full_out('*Unintentional user break!*');
        end
        br=1;
    else br=0;
    end
    
% process the tasks for the problem currently selected, eventually
% for another problems when no pause is set, callback
function execute_TB_Callback(hObject, eventdata, handles)
    global glconf;      %global configuration structure
    global glconfstat;  %status of the loaded configuration
    global glbreak;     %gloptlab break flag    
    global gloutput;    %global output structure
    
    %break check
    if get(handles.execute_TB, 'Value')==0
        %break
        set(handles.execute_TB, 'String','Execute');
        drawnow;
        glbreak=1;return;
    end

    %check intlab
    if glconfstat.intlab_running==0,
        h=warndlg('Intlab is not running, check the configuration!',...
            'Intlab missing','modal');
        set(handles.execute_TB, 'Value',0);
        uiwait(h);
        return;
    end
    
    %check if problems present
    problist=get(handles.prob_list_LB, 'String');
    pn=length(problist);
    if pn==0
        %no problems
        h=warndlg('No problems selected!','No Problems','modal');
        set(handles.execute_TB, 'Value',0);
        uiwait(h);
        return;
    end
    
    %fill in random problems in problist
    for i=1:pn
        if ~isempty(findstr(problist{i}, 'rand')) && isempty(find(problist{i}=='.', 1))
            %random problem 
            for j=1:length(handles.random)
                if strcmp(handles.random{j}.name, problist{i})
                    problist{i}=handles.random{j};
                    break;
                end
            end
            %set xbox
            problist{i}.xbox.inf=-inf+zeros(length(problist{i}.Q.int),1);
            problist{i}.xbox.sup=inf+zeros(length(problist{i}.Q.int),1);
        else
            %add path to problem name
            problist{i}=[cd filesep problist{i}];
        end
    end
    %set button label & break flag
    set(handles.execute_TB, 'String','Break');
    glbreak=0; %#ok<NASGU>
    drawnow;
    %get problemlist & startegy
    startprob=get(handles.prob_list_LB, 'Value');
    strategy.name=get(handles.strategy_file_ET, 'String');
    strategy.tasks=handles.tasks;tn=length(strategy.tasks);
    starttask=get(handles.task_list_LB, 'Value');
    %check if at the last entry in startegy & last problem->first prob
    if starttask==tn && startprob==pn,
        %at the last problem, last task, begin anew
        starttask=1;set(handles.task_list_LB, 'Value', starttask);
        startprob=1;set(handles.prob_list_LB, 'Value', startprob);
    elseif starttask~=1 && isempty(handles.P)
        %no problem loaded, begin at read task (begin of each strategy)
        starttask=1;set(handles.task_list_LB, 'Value', starttask);
    end
    if starttask==1,
        task=strategy.tasks{1};
        %clear poblem
        handles=clear_problem(handles);
        %clear diary
        if task.argin.cprt, clear_diary(handles);end
        %clear matlab window
        if task.argin.cdeb, clc;end
    end
    %flags
    gra=get(handles.graphicsout_TB, 'Value');
    pro=get(handles.proof_TB, 'Value');
    if get(handles.pause_CB, 'Value'), nopause=0;
    else nopause=2;
    end
    sta=get(handles.stat_TB, 'Value');
    %timer
    if isfield(handles,'timer') && ~isempty(handles.timer)
        %use previous timer
        timer=handles.timer;
    elseif isfield(glconf,'timer')
        %use timer setting given in glconfig
        timer=glconf.timer;
        timer.status      = 0;
        timer.ellaped     = 0;
    else
        %set timer
        timer=glcreatetimer(inf, 15, 3, 0, inf, [],[]);
    end
    %set output function
    gloutput.out=[];gloutput.outfhandle=@diary_full_out;
    gloutput.brief=[];gloutput.briefhandle=@diary_brief_out;
    % process problems
    if ~isempty(handles.P), xbox=handles.P.xbox;
    else xbox=[];
    end
    TESTENV=0;
    timer=gltimer(timer, 's');
    % process the problems
    [sols, P, xbox, probpos, newstrategy, taskpos, status, timer, ...
        certs]=glprocplist(problist, strategy, glconf.soldir, nopause, ...
        startprob, starttask, handles.P, xbox, timer, strategy, 1,...
        TESTENV, gra, gloutput.plotfigure, gloutput.plotspace, pro, ...
        glconf.proofdir, sta, glconf.statfile, handles.certs, 0);
    % save the timer
    handles.timer=timer;
    % results
    if ~isfield(P,'solstatus') || glsolstatus(P,'NORMAL')
        if isfield(P,'Q')
            handles.P=P;
            handles.P.xbox=xbox;
            handles.certs=certs;
            % new graphics output flag
            if gra,
                set(handles.redraw_PB, 'Enable', 'on');
                set(handles.zoom_TB,'Enable','on','Value',0,...
                    'String','Zoom In');
            end
            % refresh xbox
            set_xbox(handles);
            % set strategy
            if ~strcmp(strategy.name,newstrategy.name)
                % new strategy loaded in glprocplist, set it
                handles=load_strategy(handles, newstrategy.name);
                strategy.tasks=handles.tasks;
                strategy.name=newstrategy.name;
            end
            % sound
            if status==-2, glsound('break');
            elseif status==-1, glsound('pause');
            end
        else
            glsound('criterror');
        end
    end
    % set gui
    set(handles.prob_list_LB, 'Value', probpos);
    set(handles.task_list_LB, 'Value', sum(taskpos));
    
    % check the buttons
    button_check(handles);
    
    % set problem info & sync probs & files
    if isfield(handles.P,'Q'), set_problem_info(handles, handles.P); end
    probs_files_sync(handles,0);
    
    % set execute button
    set(handles.execute_TB, 'String','Execute','Value',0)
    glbreak=0;
    
    % update handles structure
    guidata(hObject, handles);
    
%--------------------------------------------------------------------------
% Proofs
%--------------------------------------------------------------------------
% proof enable on/off callback
function proof_TB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    global glconf;%global configuration variable
    glproof_gui;
    if glconf.proo.enabled, 
        set(hObject, 'Value', 1, 'String', 'Turn Off');
        set(handles.proof_T,'String','Proof: ON');
    else
        set(hObject, 'Value', 0, 'String', 'Turn On');
        set(handles.proof_T,'String','Proof: OFF');
    end
    
% browse the saved proofs callback
function browse_proofs_PB_Callback(hObject, eventdata, handles)
    %break check
    if break_strategy(handles), return; end
    %init
    global glconf;%global configuration variable
    prt=get(handles.diary_TB, 'Value');
    %browse proofs
    if exist(glconf.proofdir,'dir')==7 && ...
            exist([glconf.proofdir 'index.html'],'file')==2,
        [status,result] =glsystem(['open ' glconf.proofdir 'index.html']);
        if prt, disp(result); end
    else
        h=warndlg('No proofs found!','No proofs to browse','modal');
        uiwait(h)
    end
    
% --- Executes on button press in remove_proof_PB.
function remove_proof_PB_Callback(hObject, eventdata, handles)
    %break check
    if break_strategy(handles), return; end
    %init
    global glconf;%global configuration variable
    if exist(glconf.proofdir,'dir')==7 && ...
            exist([glconf.proofdir 'index.html'],'file')==2,
        pname=glremoveproof_gui('Proofdir',glconf.proofdir);
        if ~isempty(pname)
            h = questdlg(['Remove the automatic proof for the problem: '...
                pname ' ?'],'Remove Proof','Yes','No','No');
            if strcmp(h,'Yes')
                glremoveproof(pname, glconf.proofdir);
            end
        end
    else
        h=warndlg('No proofs found!','No proofs to remove','modal');
        uiwait(h)
    end
    
% remove all saved proofs callback
function remove_all_proofs_PB_Callback(hObject, eventdata, handles)
    %break check
    if break_strategy(handles), return; end
    %init
    global glconf;%global configuration variable
    if exist(glconf.proofdir,'dir')==7 && ...
            exist([glconf.proofdir 'index.html'],'file')==2,
        h = questdlg('Are you sure that you want to delete all proofs?',...
            'Remove Proofs','Yes','No','No');
        if strcmp(h,'Yes'), delete([glconf.proofdir '*']);end
    else
        h=warndlg('No proofs found!','No proofs to remove','modal');
        uiwait(h);
    end

function handles=convert(handles)
    global glconfstat;
    prt=get(handles.diary_TB, 'Value');
    fnames = get(handles.files_LB,'String');
    fname = fnames{get(handles.files_LB,'Value')};
    [d, name, ext] = fileparts(fname);
    task.argin.source=fname;
    task.argin.sourcedir=cd;
    task.argin.converter=[];
    switch ext,
        case '.dag',
            if isunix,
                task.argin.converter={'dag2def'};
                task.argin.target={[name '.def']};
            end
        case '.sml'
            task.argin.converter=[{'sml2glb'};{'sml2def'}];
            task.argin.target=[{[name '.glb']};{[name '.def']}];
        case '.mod'
            task.argin.converter=[{'sml2glb'};{'sml2def'}];
            task.argin.target=[{[name '.glb']};{[name '.def']}];
            if isunix,
                task.argin.converter=[{'mod2def'};task.argin.converter];
                task.argin.target=[{[name '.def']};task.argin.target];
            end
        case '.glb'
            task.argin.converter=[{'glb2sml'};{'glb2mod'}];
            task.argin.target=[{[name '.sml']};{[name '.mod']}];
        case '.def'
            task.argin.converter=[{'def2sml'};{'def2mod'}];
            task.argin.target=[{[name '.sml']};{[name '.mod']}];
    end
    if ~isempty(task.argin.converter)
        out=glconvert_gui('Task',task);
        if ~isempty(out)
            if prt,
                diary_full_out(out{1});
                diary_brief_out(out{1});
            end
            res=out{2}; % filename of the converted file
            if ~isempty(res)
                handles=get_directory(handles);
                files=get(handles.files_LB,'String');
                ind=glfindstr(files, res);
                if ~isempty(ind)
                    set(handles.files_LB, 'Value',ind(1));
                    handles=set_file(handles);
                end
                glsound('tfinish');
            else glsound('error');
            end
        end
    elseif prt,
        diary_full_out('No converter for this file type has been found!');
        diary_brief_out('No converter for this file type has been found!');
        glsound('error');
    end
   
    
%--------------------------------------------------------------------------
% Conversion of .sml .mod .dag, .def or .glb files
%--------------------------------------------------------------------------
% convert a .dag, .glb, .mod or sml file to a .glb, .mod or .sml file
function convert_PB_Callback(hObject, eventdata, handles)
    % convert
    handles=convert(handles);
    % Update handles structure
    guidata(hObject, handles);
    
%--------------------------------------------------------------------------
% Editing .mod or .sml files
%--------------------------------------------------------------------------
% create a new .mod file and edit it, callback
function new_file_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    gleditor_gui('OutputFile', 'untitled.mod');
    handles=get_directory(handles);
    % Update handles structure
    guidata(hObject, handles);
    
% edit the .mod file selected in the file listbox, callback
function edit_file_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    fnames = get(handles.files_LB,'String');
    fname=fnames{get(handles.files_LB,'Value')};
    if ~isempty(strfind(fname, '.mod')) || ~isempty(strfind(fname, '.sml'))
        gleditor_gui('InputFile', fname);
    end

%--------------------------------------------------------------------------
% View or draw a .dag file
%--------------------------------------------------------------------------
% view a .dag file in the editor
function view_dag_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    fnames = get(handles.files_LB,'String');
    fname=fnames{get(handles.files_LB,'Value')};
    if ~isempty(strfind(fname, '.dag'))
        gleditor_gui('InputFile', fname, 'Edit', 'off','FontSize', 10);
    end

% draw a .dag file by converting into a .dot and then a .ps or a .pdf file.
function draw_dag_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    global glconf;%global configuration variable
    if ~isfield(glconf,'cocopath') || ...
        ~(isfield(glconf,'psview') || isfield(glconf,'pdfview'))
        return
    end
    prt=get(handles.diary_TB, 'Value');
    fnames = get(handles.files_LB,'String');
    fname=fnames{get(handles.files_LB,'Value')};
    fname=fname(1:find(fname=='.')-1);
    status=0;
    %convert .dag to .dot
    if ~glnewer([fname '.dot'],[fname '.dag'])
        [status,result]=glsystem(['dag2dot ' fname '.dag']);
        if prt,disp(result);end;
        if ~status && prt,
            disp([fname '.dag converted to ' fname '.dot']);
        elseif prt,
            disp('dot converison failed!');
        end
    end
    if ~status
        %convert .dot to .ps
        if ~glnewer([fname '.ps'],[fname '.dot'])
            [status,result]=glsystem(['dot2ps ' fname '.dot']);
            if prt,disp(result);end;
            if ~status && prt,
                disp([fname '.dot converted to ' fname '.ps']);
            elseif prt,
                disp('ps converison failed!');
            end
        end
        if ~status,
            if isfield(glconf,'psview')
                %view .ps
                [status,result]=glsystem(['open ' fname '.ps']);
                if prt,disp(result);end;
            elseif isfield(glconf,'pdfview')
                %convert .ps to .pdf
                if ~glnewer([fname '.pdf'],[fname '.ps'])
                    [status,result]=glsystem(['ps2pdf ' fname '.ps']);
                    if prt,disp(result);end;
                    if ~status && prt,
                        disp([fname '.ps converted to ' fname '.pdf']);
                    elseif prt,
                        disp('pdf converison failed!');
                    end
                end
                if ~status
                    %view .pdf
                    [status,result]=glsystem(['open ' fname '.pdf']);
                    if prt,disp(result);end;
                end
            end
        end
    end
   
    
%--------------------------------------------------------------------------
% Configuration
%--------------------------------------------------------------------------
% edit the configuration file in the editor callback
function edit_conf_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    prt=get(handles.diary_TB, 'Value');
    fname = get(handles.conffile_ET,'String');
    confdir=[fileparts(which('Gloptlab.m')) filesep];
    if exist([confdir fname],'file')==2
        fname=gleditor_gui('Path', confdir, 'InputFile', fname, ...
            'FontSize', 9);
        if ~isempty(fname)
            ind=find(fname==filesep);ind=ind(end)+1;
            fname=fname(ind:end);
            set(handles.conffile_ET,'String',fname);
            handles=loadconf(fname, handles);
            handles=refresh_tasklist(handles);
            if prt,diary_full_out(['*Configuration file '...
                    fname ' initialized!*']);end
        end
    end
    % Update handles structure
    guidata(hObject, handles);
    
% use the current configuration callback
function use_conf_PB_Callback(hObject, eventdata, handles) 
    if break_strategy(handles), return; end
    prt=get(handles.diary_TB, 'Value');
    fname = get(handles.conffile_ET,'String');
    handles=loadconf(fname, handles);
    handles=refresh_tasklist(handles);
    if prt,diary_full_out(['*Configuration file '...
                    fname ' initialized!*']);end
    % Update handles structure
    guidata(hObject, handles);

% load the configuration
function handles=loadconf(fname, handles)
    global glconf;
    %initialize the configuration with extra checks
    glinitconf([fileparts(which('Gloptlab.m')) filesep fname],1) 
    %set graphics axe ranges
    if isfield(glconf, 'refboxran')
        set(handles.xrange_ET, 'String',{num2str(glconf.refboxran)});
    end
    glconf.mainfig=handles.figure1;
    
    
%--------------------------------------------------------------------------
% Profiler
%--------------------------------------------------------------------------
% turn the profining on/off callback
function profile_TB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    state=get(hObject, 'Value');
    if state,
        set(hObject, 'String', 'Finish');
        set(handles.profiler_T,'String','Profiler: ON');
        profile on;
    else
        set(hObject, 'String', 'Start');
        set(handles.profiler_T,'String','Profiler: OFF');
        profile viewer;
    end
    
%--------------------------------------------------------------------------
% Statistics
%--------------------------------------------------------------------------
% turn the statistics on/off callback
function stat_TB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    if get(hObject, 'Value'),
        set(hObject, 'String', 'Turn Off');
        set(handles.statistic_T,'String','Statistic: ON');
    else
        set(hObject, 'String', 'Turn On');
        set(handles.statistic_T,'String','Statistic: OFF');
    end
    
% clear the statistic callback 
function clear_stat_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    global glconf;
    if exist(glconf.statfile,'file')==2,
        h = questdlg(['Are you sure that you want to delete all '...
            'statistics?'],'Remove Proofs','Yes','No','No');
        if strcmp(h,'Yes'), delete(glconf.statfile);end
    else
        h=warndlg('No statistics found!','No statistic','modal');
        uiwait(h);
    end

% browse the statistic callback
function display_stat_PB_Callback(hObject, eventdata, handles)
    if break_strategy(handles), return; end
    global glconf;
    fname=glconf.statfile;
    if(exist(fname,'file')==2)
        mode=get(handles.configure_stat_PM, 'Value');
        stat=glshowstatistics(fname, mode);
        [d, name, ext] = fileparts(fname);
        gleditor_gui('Filename', [name ext], 'Edit', 'off', 'Text',...
            stat,'FontSize', 10);
    else
        h=warndlg('No statistics found!','No statistic','modal');
        uiwait(h);
    end
