%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glgetguidata.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function glgetguidata(handles, lang, ldir)
%
% Set the text in the gui by using the data files. There are two
% type of files: str and tts files. The str files contain the String
% attribute of the gui elements while the tts files the TooltipStrings.
% e.g. the glmain_gui_tts_EN.txt contains the english tooltip string for
% the glmain_gui.fig.
%
% handles      handles structure of the gui
% lang         2 x 1 language identifier string, e.g. 'EN', 'DE' 
% ldir         the directory containig the language files 

function glgetguidata(handles, lang, ldir)

global glconf;    % global configuration variable

% argchek
if nargin <1, error('glgetguidata: Not enough arguments!');end
if nargin <2, lang=[]; end
if nargin <3, ldir=[]; end

% init
[d,fname]=fileparts(get(handles.figure1, 'FileName'));
if isempty(lang)
    if isfield(glconf,'language'), lang=glconf.language; 
    else lang='en';
    end
end
lang=lower(lang);
if isempty(ldir)
    ldir=[d filesep 'GuiData' filesep upper(lang) filesep ];
end

% modify the gui components
strf=[ldir fname '_str_' lang '.txt'];
if exist(strf,'file')==2,
    % load and set the text into the gui
    data=glload(strf);
    fnams=fieldnames(data);
    fn=length(fnams);
    for i=1:fn
        fn=fnams{i};
        if isfield(handles,fn)
            set(handles.(fn),'String',data.(fn));
        end
    end
end
ttff=[ldir fname '_tts_' lang '.txt'];
if exist(ttff,'file')==2,
    % load and set the tooltip text into the gui
    data=glload(ttff);
    fnams=fieldnames(data);
    fn=length(fnams);
    for i=1:fn
        fn=fnams{i};
        if isfield(handles,fn)
            set(handles.(fn),'TooltipString',data.(fn));
        end
    end
end
hlpf=[ldir fname '_hlp_' lang '.txt'];
if exist(hlpf,'file')==2,
    % load and set the help text and set the callbacks
    data=glload(hlpf);
    fnams=fieldnames(data);
    fn=length(fnams);
    for i=1:fn
        fn=fnams{i};
        if isfield(handles,fn)
            set(handles.(fn),'ButtonDownFcn',...
                {@glguiclick_callback,data.(fn)});
        end
    end
end