%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glguichk.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% handles = glguichk(handles)
%
% Set the components of the gui as enabled or disabled by determining the
% input arguments of the corresponding function(s).
%
% handles       handles structure of a task gui. The enabled status of the 
%               fields .(vname)_ are set exculding the ones given in the
%               NOCHANGE vector.
%
% handles       updated handles structure.

function handles = glguichk(handles)

% theese fields of the handles contain a '_' but their enabled status
% remain unchanged.
NOCHANGE=[{'ok_PB'},{'cancel_PB'},{'method_PM'}];

% argcechk
if nargin<1, error('glguichk: Not enough arguments!');end

% get the name of the task = name of the the figure
tname=get(handles.figure1, 'Name');
% change taskname if the task is a Begin-End task
if length(tname)>6 && strcmp(tname(1:6),'Begin '),
    tname=tname(7:end);
end
tname=lower(tname);

% get the field names of the gui and delete the ones which are never
% changed
fnams=fieldnames(handles);
ind=[];
for i=1:length(fnams)
    for j=1:length(NOCHANGE)
        if strcmp(fnams{i},NOCHANGE{j})
            ind=[ind i]; break;
        end
    end
end
fnams(ind)=[];
    
% get name of the task function
if isfield(handles, 'method_PM')
    % this task contains a method the task function name is
    % gl(taskname)_(method)
    methods=get(handles.method_PM, 'String');
    method=methods{get(handles.method_PM, 'Value')};
    if ~strcmp(method,'NONE'), funname=['gl' tname '_' method];
    else return;
    end
else funname=['gl' tname];
end

if exist(funname,'file')==2
    % get all possible input argument names for the task by calling the
    % task function without input arguments
    ar=feval(funname);allargs=[ar{1};ar{2}];

    % enable or disable the fields of the gui.
    for i=1:length(fnams)
        fname=fnams{i};
        if strcmp(fname(1:2),'gl')
            % external method, remove the method name.
            ind=find(fname=='_');
            if ~isempty(ind),sname=fname(ind(1)+1:end);end
        else sname=fname;
        end
        ind=strfind(sname,'_');
        if length(ind)==1
            found=false;sname=sname(1:ind-1);
            for j=1:length(allargs)
                if strcmp(sname,allargs{j}),
                    found=true;break;
                end
            end
            if found, set(handles.(fname),'Enable' , 'on')
            else set(handles.(fname),'Enable' , 'off')
            end
        end
    end
end
