%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glguiout.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function task=glguiout(handles)
%
% Get all task data from a task GUI. Return a new task (or a cell of two
% tasks in case of a Begin-End task) corresponding to the GUI.
%
% handles       handles structure of a task gui.
%               there are 3 kinds of entries which will be set:
%   .(vname)_CB This is a checkbox, its value will be set in the
%               correspondig field: task.argin.(vname)
%   .(vname)_ET This is a edit text, its value will be set in the
%               correspondig field: task.argin.(vname)
%   .(vname)_PM This is a popup menu, its value will be set in the
%               correspondig field: task.argin.(vname).
%           
% task          output task structure or cell ( in case the task starts
%                   with a 'Begin' ) 
%  .name        name of the task == name of the GUI figure
%  .argin       input argument structure containig various input args,
%               all theese are extracted from the handeles

function task=glguiout(handles)

global glconf;    % global configuration variable

% argcechk
if nargin<1, error('glguiout: Not enough arguments!');end

% get name taskname = name of the figure
task.name=get(handles.figure1, 'Name');

% determine if the task is a Begin-End task or not
ctask=length(task.name)>6 && strcmp(task.name(1:6),'Begin ');

% init tasknam and prefix
task.argin=[];fnams=fieldnames(handles);
if ctask,
    % Begin-End task
    tstr=task.name(7:end);  % type string like 'Split'
    tnam=lower(tstr);       % name of the task in lowercase letters
else tnam=lower(task.name); % name of the task in lowercase letters
end
tpre=tnam(1:4);             % task perfix, used in argin

% get various fields for the task argin
for i=1:length(fnams)
    fn=fnams{i};
    if length(fn)>3 && fn(end-2)=='_' && ...
            strcmp(get(handles.(fn), 'Enable'),'on');
        vnam=fn(1:end-3);           % full variable name
        type=fn(end-1:end);         % variable type
        gfield=handles.(fn);% actual gui field
        % handle the different types
        switch type
            case 'CB'
                value=logical(get(gfield, 'Value'));
                task.argin.(vnam)=value;
            case 'RB'
                value=get(gfield, 'Value');
                if value,
                    ind=find(vnam=='_');
                    if ~isempty(ind), vnam=vnam(1:ind(1)-1); end
                    value=get(gfield, 'String');
                    task.argin.(vnam)=value;
                end
            case 'ET'
                value=get(gfield, 'String');
                task.argin.(vnam)=value;
            case 'EV'
                value=get(gfield, 'String');
                task.argin.(vnam)=str2num(value);
            case 'PM'
                if strcmp(vnam(1:2),'gl')
                    % external method, remove the method name.
                    ind=find(vnam=='_');
                    if ~isempty(ind),vnam=vnam(ind(1)+1:end);end
                end
                values=get(gfield, 'String');
                if iscell(values), value=values{get(gfield, 'Value')};
                else value=values;
                end
                if strcmp(value, 'NONE'), task=[]; return; end
                task.argin.(vnam)=value;
            otherwise
                continue;
        end
    end
end
% set the config
glconf.(tpre)=task.argin;

% create the End task in case of Begin-End task
if ctask,
    task2.name=['End ' tstr];     % correspondig end str:'End Split'
    task=[{task};{task2}];
end

% debug
deb=0;
if deb,
    fnams=fieldnames(glconf.(tpre));
    for i=1:length(fnams)
        val=glconf.(tpre).(fnams{i});
        if isnumeric(val), txt=num2str(val);
        else txt=val;
        end
        disp([tpre '.' fnams{i} '='  txt ' (' class(val) ')']);
    end
end
