%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%% glsetguipos.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function glsetguipos(fh, corner, posy, posx)
%
% Position the figure with the given handle relative to the main gui
% position, which is saved in glconf.mainguipos.
%
% fh        handle of the figure
% corner    2 x 1 corner identifier string choosen from ur,ul,lr,ll
% posy      y position relative to the main gui:'u','l','u+..','l-..' etc.
% posx      x position relative to the main gui:'r','l','r+..','l+..' etc.
%           Example: glsetguipos(fh, 'ur', 'u', 'l+0.01') sets the postion
%           of the figure that the upper right coner matches to the upper
%           left corner of the gui, where it is shifted to the right by
%           0.01 times the width of the main gui.

function glsetguipos(fh, corner, posy, posx)

global glconf;    % global configuration variable

% argchek
if nargin <4, error('glsetguipos: Not enough arguments!');end

% see if values are stored in guiprefs
cornerp = glgetguipref(fh,'corner');
posyp = glgetguipref(fh,'posy');
posxp = glgetguipref(fh,'posx');

% if we have no values, set them
if (isempty(cornerp) || isempty(posxp) || isempty(posyp))
    glsetguipref(fh,'corner',corner);
    glsetguipref(fh,'posx',posx);
    glsetguipref(fh,'posy',posy);
else % else use stored ones
    corner = cornerp;
    posy = posyp;
    posx = posxp;
end

% init
set(fh,'Units','pixels');

pos=get(fh,'Position');
w=pos(3);h=pos(4);
mpos=get(glconf.mainfig,'Position');
mx=mpos(1);my=mpos(2);mw=mpos(3);mh=mpos(4);

% compute the y coodinate and the heigth
shifted=length(posy)~=1;
if ~shifted, pc=0;
else pc=str2num(posy(2:end))*mh;
end
if posy(1)=='l', py=my+pc;
elseif posy(1)=='u', py=my+mh+pc;
else error('glsetguipos: Wrong posy format!')
end
if corner(1)=='l', pos(2)=py;
elseif corner(1)=='u', pos(2)=py-h;
else error('glsetguipos: Wrong corner format!')
end
% compute x coodinate and the width
shifted=length(posx)~=1;
if ~shifted, pc=0;
else pc=str2num(posx(2:end))*mw;
end
if posx(1)=='l', px=mx+pc;
elseif posx(1)=='r', px=mx+mw+pc;
else error('glsetguipos: Wrong posx format!')
end
if corner(2)=='l', pos(1)=px;
elseif corner(2)=='r', pos(1)=px-w;
else error('glsetguipos: Wrong corner format!')
end

set(fh,'Position',pos);