%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glplot_callback.m %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% glplot_callback(hObject, eventdata, ind, J)
%
% Callback for the variable range textfields created in the 
% glplot_init function.
%
% hObject   the handle of the textbox
% eventdata event data (empty)
% ind       this is the callback for the range of the variable x_ind
% J         2 x n-1 matrix indicating that one axis in the subplots with 
%           handles A(1,:) corresponds to this variable. The second row 
%           A(2,:) indicates to which axis (1 for horizontal or 2 for 
%           this variable corresponds in each plot.

function glplot_callback(hObject, eventdata, ind, J) %#ok<INUSL>
global gloutput;    % global output structure

ran=str2num(get(hObject,'String'));
if ~isempty(ran) && all(size(ran)==[1,2]) && ran(1)<ran(2)
    % rescale the axis in each subplot
    set(hObject, 'Foreground','k');
    for i=1:size(J,2)
        if J(2,i)==1, set(J(1,i),'XLim',ran);
        else set(J(1,i),'YLim',ran);
        end
    end
    % set the referece box in the global graphics configuration
    if isfield(gloutput,'graphics')
        gloutput.graphics.refbox.inf(ind)=ran(1);
        gloutput.graphics.refbox.sup(ind)=ran(2);
    end
else
    % error in the input format of the range
    set(hObject, 'Foreground','r');
end

