%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glfindspars.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function Q=glfindspars(Q);
% finds the sparsity pattern of Hessian of Lagrangian for the problem
%
% Q                     structure encoding the cut
%      .A               1 x (nx+nx^2) sparse matrix of the cut coefficients
%        .inf,.sup      bounds for coefficients
%      .int             characteristic vector of integer variables
%
% Q                     structure encoding the cut
%      .A               1 x (nx+nx^2) sparse matrix of the cut coefficients
%        .nz            incidence matrix
%      .hess            sparsity pattern of Hessian of Lagrangian

function Q=glfindspars(Q)

% argchk
if nargin<1
    error('glfindspars: Not enough arguments!');
end

nx=length(Q.int);
Q.A.nz=(Q.A.inf~=0 | Q.A.sup~=0);
Q.hess=(sum(Q.A.nz(:,nx+1:nx*(nx+1)),1)>0);
Q.hess=reshape(Q.hess,nx,nx);
Q.hess=Q.hess|Q.hess';