%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glprobinfo.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function info=glprobinfo(P);
% supplies information about the problem P
%  P            problem definition
%
%  info         information structure
%   .name       problem name
%   .desc       problem description
%   .nx         number of variables
%   .no         number of original variables
%   .ni         number of integer variables
%   .nc         number of constraints
%   .ne         number of nonquadratic basis functions
%   .nu         number of univariate basis functions
%   .nB         number of basis functions (all)
%   .nA         number of constants in P.Q.A

function info=glprobinfo(P)
info.name=P.name;
info.desc=P.desc;
info.nx=length(P.Q.int);
info.no=P.Q.nxo;
info.ni=nnz(P.Q.int);
[info.nc,info.nB]=size(P.Q.A.inf);
info.ne=info.nB-info.nx-info.nx^2;
info.nu=length(P.uni);
info.nA=nnz(P.Q.A.inf | P.Q.A.inf);