%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glupdate.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function glupdate(fname, varname, value)
%
% Update an exitsting file which was saved by the glsave method by 
% owerwriting the value of the given variables.
%
% fname      existing file saved by glsave, with full path
% varname    a variable name which exits in the given file.
% value      a constant, matrix or cell or struct value which will replace 
%            the old one.

function glupdate(fname, varname, value)

% argcheck
if nargin < 3
    error('glupdate: not enough arguments!');
end

% read the file
fid=fopen(fname,'r');
if fid==-1
    error(['glupdate: could not open the file: ' fname '!']);
    return
end
strc=cell(0);
vnlen=length(varname);
i=1;rind=0;
while 1
    tline = fgetl(fid);
    if length(tline)>vnlen && strcmp(tline(1:vnlen),varname), rind=i; end
    if ~ischar(tline), break, end
    strc=[strc; {tline}];
    i=i+1;
end
fclose(fid);
% check the row index which contains the variable
if rind==0
    disp(['glupdate: could not find the variable: ' varname ...
        ' in the file: ' fname '!']);
    return
end
% convert the new value
ind=sort([0 find(varname=='.')],2,'descend');
i=vnlen;var=value;
for j=ind
    k=var;var=[];
    var.(varname(j+1:i))=k;
    i=j-1;
end
s=glstruct2scell(var);
% replace the row
row=strc{rind};ind=find(row==';');
strc{rind}=[s{1} row(ind(end)+1:end)];
% write the file
fid=fopen(fname,'w');
if fid == -1, error(['glupdate: Filename wrong:' outfname]); end
for i=1:length(strc)
    fprintf(fid,'%c',strc{i});
    fprintf(fid,'\n');
end
fclose(fid);
