%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glbreak.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glbreak(argin)
%
% Break the execution of the tasklist, a multiline (while or split), jump
% to the postprocessing step or finish the strategy.

% * needed argin structure input fields *
% mode      'break tasklist', 'break multiline', 'postprocess' or 
%           'finish strategy'
%
% * optional argin structure input fields *
% prt       text output flag       0 = no output 1 = output
% deb       debug output flag      0 = no output 1 = output
% gra       graphics output flag   0 = no output 1 = output
% pro       certificate output     0 = no cert   1 = make cert
%
% * argout structure fields *
% status        = 0 proceed with the next task
%               = 1 stop the strategy (pause or user break)
%               = 2 break this tasklist (glbreak->break tasklist, infeas)
%               = 3 break the multiline (glbreak->break multiline)
%               = 4 continue with postprocess (glbreak->postprocess,timeout)
%               = 5 finish the strategy (glbreak->finish strategy, problem 
%                   not accepted: dim too high/nonquadratic found, after 
%                   postprocess)

function argout=glbreak(argin)

global gloutput;    % global output structure

% needed input fields & determine the function name 
ninp={'mode'}; 
thisfun=mfilename;

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'}];
dval=[{  0  };{  0  };{  0  };{  0  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

argout.status=0;
if strcmp(argin.mode,'break tasklist'),         argout.status=2;
elseif strcmp(argin.mode,'break multiline'),    argout.status=3;
elseif strcmp(argin.mode,'postprocess'),        argout.status=4;
elseif strcmp(argin.mode,'finish strategy'),    argout.status=5;
else error([thisfun ': Unknown mode:' argin.mode ' found!']); 
end
if argin.prt,
    gloutput.out=[gloutput.out ; {['*** BREAK: mode=' argin.mode ' ***']}];
end
