%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gldispcert.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function gldispcert(cert, xbox);
% displays the certifcate :
% p(x)=||B(1 x)'|| - (1 xu-x x-xl)Z(1 xu-x x-xl)' - zEq(x)
%
% cert          certificate structure
%    .B         nx+1 x nx+1 permuted cholesky factor of G, G=B'B
%    .Z         2nx+1 x 2nx+1 triangular matrix
%    .z         nc dim vector
% xbox          bound constraints

function gldispcert(cert, xbox)

global glconf; %global configuration variable
PREC=1e-6;     %percision displaying 1 instead of x in [1-PREC,1+PREC]

% ARGCHECK %
if nargin<1, error('gldispcert: Not enough arguments!');end

%set constants from config
if isfield(glconf,'tiny'),PREC=glconf.tiny;end

%init
%G=tril(cert.G)+triu(cert.G)'-diag(diag(cert.G));
B=cert.B;
Z=tril(cert.Z)+triu(cert.Z)'-diag(diag(cert.Z));
nx=size(B,1)-1;
nZ=size(Z,1);ny1=(nZ-1)/2;
nz=length(cert.z);

%create variable string cell
Bvnams=cell(nx+1,1);Bvnams{1}='';
for i=1:nx
    Bvnams{i+1}=['x_' num2str(i)];
end
Zvnams=cell(nZ,1);Zvnams{1}='';
for i=1:ny1
    Zvnams{i+1}=['(x_' num2str(i) '-' num2str(xbox.inf(i)) ')'];
    Zvnams{ny1+i+1}=['(' num2str(xbox.sup(i)) '-x_' num2str(i) ')'];
end
out='';
% display ||B(1 x)||
for i=1:nx+1
    inds=find(B(i,:) ~= 0);
    if isempty(inds)
        continue;
    end
    out=[out '+('];
    for j=inds
        c=B(i,j);
        if c == 0, continue;end
        %print constant
        if -1-PREC<c && c<-1+PREC, out=[out '-'];
        else
            if  c>0 && j~=inds(1), out=[out '+'];end
            if 1-PREC>c || c>1+PREC || j==1, out=[out num2str(c)];end
        end
        %print variables
        out=[out Bvnams{j}];
    end
    out=[out ')^2'];
end
% constant part
if ~isempty(out),out=[ 'p(x) = ' out(2:end) ' + ' num2str(-Z(1,1))];
else out=[ 'p(x) = ' num2str(-Z(1,1))];end
% display (1 xu-x x-xl)Z(1 xu-x x-xl)'
for i=2:nZ
    inds=find(Z(i,:) ~= 0);
    if isempty(inds)
        continue;
    end
    for j=inds
        c=-Z(i,j);
        if c == 0, continue;end
        %print constant
        if -1-PREC<c && c<-1+PREC,out=[out '-'];
        else
            if c>0, out=[out '+'];end
            if 1-PREC>c || c>1+PREC, out=[out num2str(c)];end
        end
        %print variables
        if i==j, out=[out Zvnams{i} '^2'];
        else out=[out Zvnams{i} Zvnams{j}];end
    end
end
%Display zEq(x)
for i=1:nz
    c=-cert.z(i);
    if c == 0, continue;end
    %print constant
    if -1-PREC<c && c<-1+PREC,out=[out '-'];
    elseif 1-PREC<c && c<1+PREC,out=[out '+'];
    else 
        if c>0, out=[out '+'];end
        out=[out num2str(c)];
    end
    %print constraint indicator
    out=[out '[' num2str(i) ']'];
end
disp(out)