%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glfeasdistfun.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [fd, grad, fdv]=glfeasdistfun(x);
%
% Feasibility distance function.
%
% gllocvars     this global variable must contain mid(A) and F, for 
%               Aq(x) \in F
% x             nx x 1 vector to evaluate the constraint at 
%
% fd            scalar feasibility distace 
% grad          nx x 1 gradient vector
% fdv           nc x 1 feasibility distance vector: 
%               fdv_i=f(x)-Fu if f(x)>Fu, f(x)-Fl if f(x)< Fl, 0 otherwise

function [fd, grad, fdv]=glfeasdistfun(x)

global gllocvars; % global structure which must contain A and F
global glgradhelp;% global gradient helper

% argcheck
if nargin <1, error('glfeasdistfun: Not enough arguments!'); end
if ~isfield(gllocvars,'A') || isempty(gllocvars.A)
    error('glfeasdistfun: A must be passed in gllocvars!')
elseif ~isfield(gllocvars,'F') || isempty(gllocvars.F)
    error('glfeasdistfun: F must be passed in gllocvars!')
end
    
% init
nc=size(gllocvars.A,1); % number of constraints

% compute fx
fx=gllocvars.Al*x;xs=x';
for i=1:nc,
    fx(i)=fx(i) + xs*gllocvars.Aq{i}*x;
end

% old code
% fx=zeros(nc,1);xs=x';indl=1:nx;indq=nx+1:nA;
% for i=1:nc,
%     fx(i)=gllocvars.A(i,indl)*x + xs*reshape(gllocvars.A(i,indq),nx,nx)*x;
% end

% old code
% fx=zeros(nc,1);
%  for i=1:nc,
%       fx(i)=[1;x]'*[zeros(1,nx+1);[gllocvars.A(i,1:nx)' ...
%           reshape(gllocvars.A(i,nx+1:end),nx,nx)]]*[1;x];
%  end
% fxold=fx
% compute the feasibility distance
f1=[fx-gllocvars.F.sup; (x-gllocvars.xb.sup)];
f2=[fx-gllocvars.F.inf; (x-gllocvars.xb.inf)];
% signed nc x 1 feasibility distance vector: 
% fdv_i=f(x)-Fu if f(x)>Fu, f(x)-Fl if f(x)< Fl, 0 otherwise
fdv=zeros(size(f1));
fdv(f1>0)=f1(f1>0);
fdv(f2<0)=f2(f2<0);
% feasibility distance
fd=sum(fdv.*fdv)/2;
if nargout>1
    % compute the nx x 1 gradient vetor: Aq(x)'*fdv_generalc + fdv_boundc
    grad=glderiv(gllocvars.A, x, fdv(1:nc,1))+ fdv(nc+1:end,1);
elseif ~isempty(glgradhelp)
    % compute the nx x 1 gradient vetor: Aq(x)'*fdv_generalc + fdv_boundc
    % and set it in the gradient helper
    glgradhelp.x=x;
    glgradhelp.grad=glderiv(gllocvars.A, x, fdv(1:nc,1))+ fdv(nc+1:end,1);
end
