%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glfeasdistgrad.m %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function grad=glfeasdistgrad(x);
%
% Gradient of the feasibility distance function. The global variable
% glgradhelp is used to get the gradient information from the glfeasdistfun
% function directly.
%
% glgradhelp    gloabl gradient helper (can be empty)
%  .x           the last point evaluated
%  .grad        the gradient at x
% x             nx x 1 vector to evaluate the gradient at 
%
% grad          nx x 1 gradient vector

function grad=glfeasdistgrad(x)

global glgradhelp;% global gradient helper

% argcheck
if nargin <1, error('glfeasdistgrad: Not enough arguments!'); end

% get the gradient
if ~isempty(glgradhelp) && isfield(glgradhelp,'x') && all(glgradhelp.x==x)
    % the gradient was already computed by the glfeasdistfun function
    % at this point.
    grad=glgradhelp.grad;
else
    % compute the gradient
    glgradhelp=1;
    [fd, grad]=glfeasdistfun(x);
end