%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinsol_intlab.m %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [x, F]=gllinsol_intlab(A, F, x, vind, prt)
%
% Constraint propagation for a linear CSP,
% Ax \in \F, A \in \A, x \in \x
%
% A             nc x nx constraint coeff matrix
%     .inf,.sup outer enclosures
% F             nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%     .inf,.sup outer enclosures
% vind          not used
% prt           text output flag       0 = no output 1 = output
%
% x             nx x 1 new bound constraints
% F             nc x 1 new bounds of general constraints

function [x, F]=gllinsol_intlab(A, F, x, vind, prt)

% argcechk
if nargin<3, error('gllinsol_intlab: Not enough arguments!');end
if nargin<4, vind=[];end %#ok<NASGU>
if nargin<5, prt=0;end %#ok<NASGU>

% init
nx=length(x.inf);       % number of vars
nc=length(F.inf);       % number of constraints

% checks
if any(size(A.inf)~=[nc,nx]),
    gldisp('Asize',size(A),'nc',nc,'nx',nx);
    error('gllinsol_intlab: Dimension mismatch!');
end

% init
xn=infsup(x.inf,x.sup); A=infsup(A.inf,A.sup);

% Fn=F \cut (\A\x)
Fn=glimult(A,xn);Fn=infsup(max(Fn.inf,F.inf),min(Fn.sup,F.sup));

% invert
y=verifylss(A,Fn);

% new x
x.inf=max(x.inf,y.inf);x.sup=min(x.sup,y.sup);

% new F=Fn \cut (\A\xn)
Fn2=glimult(A,infsup(x.inf, x.sup));
F.inf=max(Fn2.inf,Fn.inf);F.sup=min(Fn2.sup,Fn.sup);

    