%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinsol_standard.m %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [x, F]=gllinsol_standard(A, F, x, vind, prt)
%
% Constraint propagation for a linear CSP,
% Ax \in \F, A \in \A, x \in \x
%
% A             nc x nx constraint coeff matrix
%     .inf,.sup outer enclosures
% F             nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%     .inf,.sup outer enclosures
% vind          not used
% prt           text output flag       0 = no output 1 = output
%
% x             nx x 1 new bound constraints
% F             nc x 1 new bounds of general constraints

function [x, F]=gllinsol_standard(A, F, x, vind, prt)

% argcechk
if nargin<3, error('gllinprop_standard: Not enough arguments!');end
if nargin<4, vind=[];end %#ok<NASGU>
if nargin<5, prt=0;end %#ok<NASGU>

% init
nx=length(x.inf);       % number of vars
nc=length(F.inf);       % number of constraints

% checks
if any(size(A.inf)~=[nc,nx])
    gldisp('Asize',size(A),'nc',nc,'nx',nx);
    error('gllinprop_standard: Dimension mismatch!');
end

% use constraint propagation on the verified results to get new bounds
argin.P.Q.A.inf=[A.inf sparse([],[],[],nc,nx^2,0)];
argin.P.Q.A.sup=[A.sup sparse([],[],[],nc,nx^2,0)];
argin.P.Q.F=F;
argin.P.Q.int=zeros(nx,1);argin.P.Q.Bu=[];
argin.P.Q=glfindspars(argin.P.Q);argin.P.Q.gaps=[];
argin.P.basis=[];argin.P.uni=[];argin.P.opt=0;
argin.x=x;argin.bilappr='a';argin.full=1;
argin.deb=0;argin.gra=0;argin.pro=0;argin.cert=[];
argout=glpropagate_separable(argin);
x=argout.x;F=argout.F;
