%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glqback.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x,x2]=glqback(a,b,c);
% rigorous enclosure of all x>=0 with a x^2 + 2b x >=c (componentwise)
% into a union of two intervals \x and \x2 
% usually, \x2 is empty
%
% a,b,c            n x 1 vector of coefficients
% prt              output level 0=none 1=some 2=debug
% 
% x.inf,x.sup      n x 1  bounds for x (left interval in case of two)
% x2.inf,x2.sup    n x 1  bounds for right interval
%                  an empty interval gets lower bound inf
%
function [x,x2]=glqback(a,b,c,prt)

if nargin <3
    error('glqback: Not enough arguments!');
elseif nargin==3
    prt=0;
end
if prt,
    for i=1:length(a)
        out=['x_' num2str(i) ': ' num2str(a(i)) ','  num2str(b(i)) ',' num2str(c(i)) ' '];
        if a(i)>=0, out=[ out 'a>=0, ']; else out=[ out 'a<0, '];end
        if b(i)>=0, out=[ out 'b>=0, ']; else out=[ out 'b<0, '];end
        if c(i)>=0, out=[ out 'c>=0'];   else out=[ out 'c<0'];end
        disp(out)
    end
end
    
n=size(a,1);

x.inf=zeros(n,1);
x.sup=inf+zeros(n,1);
x2.inf=x.sup;
x2.sup=x.sup;

ind=find(b>=0)';
if ~isempty(ind),
  setround(1); %round up
  for k=ind,
    if c(k)>0, 
      % case b>=0, c>0
      Delta=b(k)^2+a(k)*c(k);
      if Delta<0,
        x.inf(k)=inf;
      elseif a(k)==0 && b(k)==0, 
        x.inf(k)=inf;
      else
        z=b(k)+sqrt(Delta);
        x.inf(k)=-((-c(k))/z);
        if a(k)<0, x.sup(k)=z/(-a(k)); end;
      end
    else       
      % case b>=0, c<=0
      if a(k)<0, 
        Delta=b(k)^2+a(k)*c(k);
        z=b(k)+sqrt(Delta);
        x.sup(k)=z/(-a(k)); 
      end;
    end;
  end;
end;

ind=find(b<0)';
if ~isempty(ind),
  setround(-1); %round down
  for k=ind,
    if c(k)>0, 
      % case b<0, c>0
      if a(k)>0, 
        Delta=b(k)^2+a(k)*c(k);
        z=-b(k)+sqrt(Delta);
        x.inf(k)=z/a(k);
      else 
        x.inf(k)=inf;
      end
    else       
      % case b<0, c<=0
      Delta=b(k)^2+a(k)*c(k);
      if Delta>=0,
        z=-b(k)+sqrt(Delta);
        x.sup(k)=-(c(k)/z);
        if a(k)>0, x2.inf(k)=z/a(k); end;
      end;
    end;
  end;
end;