%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glbox2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [str, equnums]=glbox2str(box, I, mode, equnums, nextnum)
%
% convert a box x to a string representation.
%
% box          bound constraints for x (optional)
%  .inf,.sup   outer enclosure
% I            index set of selected omponents
% mode         0 = normal text (default)
%              1 = format as a html text
%              2 = compact (one line) text
% equnums      nx x 1 vector representing the current number of the
%              numerated bound constraints.
% nextnum      next equation number
%
% str          string cell representation of the box
% equnums      nx x 1 vector representing the current number of the
%              numerated bound constraints.

function [str, equnums]=glbox2str(box, I, mode, equnums, nextnum)

%HTMLCOLOR='#008F8F' ;

%argcheck
if nargin<1, error('glbox2str: Not enough arguments!');end
if nargin<2, I=1:length(box.inf);end
if nargin<3, mode=0;end
if nargin<5, equnums=[];nextnum=0;end
if nargin==4,error('glbox2str: nextnum must be supplied!');end

% init
nx=length(box.inf);
if mode==3
    % compact text
    str='';
    for i=I, 
        str=[str '[' num2str(box.inf(i)) ',' num2str(box.sup(i)) '];'];
    end
else
    str=cell(nx,1);
    % normal and html text
    for i=I
        if ~(isinf(box.inf(i)) && isinf(box.sup(i)))
            out='         ';
            if nextnum
                equnums(i,1)=nextnum;nextnum=nextnum+1;
                if mode==1
                    out=[out '(<small>' num2str(equnums(i)) '</small>) '];
                else
                    out=[out '(' num2str(equnums(i)) ') '];
                end
            end
            if mode==1,
                out=[out 'x<small><sub>' num2str(i) '</sub></small>'];
            else
                out=[out 'x_' num2str(i)];
            end
            if isinf(box.inf(i)),
                if mode==1,
                    out=[out '<B> &le; </B></FONT>' num2str(box.sup(i))];
                else
                    out=[out ' <= ' num2str(box.sup(i))];
                end
            elseif isinf(box.sup(i)),
                if mode==1,
                    out=[out '<B> &ge; </B>' num2str(box.inf(i))];
                else
                    out=[out ' >= ' num2str(box.inf(i))];
                end
            else
                if mode==1,
                    out=[out '<B> &isin; </B>'];
                else
                    out=[out ' in '];
                end
                out=[out '[' num2str(box.inf(i)) ...
                    ',' num2str(box.sup(i)) ']'];
            end
            str{i}=out;
        end
    end
    % delete empty entries
    i=1;l=nx;
    while i<=l
        if isempty(str{i}),str(i)=[];l=l-1;
        else i=i+1;
        end
    end
end