%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gleq2ineq.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [x, F, A]=gleq2ineq(x, slacks, E)
%
% Trasform the the general constraints from equality constraints to their
% inequality form by removing the slack variables.
%
% x              no+ns dim structure with bounds on x 
%   .inf,.sup    outer enclosure
% slacks         1 x ns slack variable indexes: x_(nx+i) used in the
%                non-equality constraint slack(i)
% E              nc x ((no+ns)+(no+ns)^2) sparse matrix of const coe
%   .inf,.sup    bounds for coefficients
%
% x              no dim structure with bounds on x 
%   .inf,.sup    outer enclosure
% F              nc x 1 struct with bounds for general constraints
%   .inf,.sup    outer enclosure
% A              nc x (no+no^2) sparse matrix of const coeffs
%   .inf,.sup    bounds for coefficients

function [x, F, A]=gleq2ineq(x, slacks, E)

% argcheck
if nargin<3, error('gleq2ineq: Not enough arguments!'); end

% init
ns=length(slacks);
nx=length(x.inf);
no=nx-ns;

% check
if size(E.inf,2)~=nx*(nx+1)+1,error('gleq2ineq: Dimension mismatch!');end

% get F
F.inf=-E.inf(:,1);F.sup=-E.sup(:,1);

% assign slack ranges to the Fbox
F.inf(slacks)=x.inf(no+1:nx);
F.sup(slacks)=x.sup(no+1:nx);

% remove slack entries from the x
x.inf=x.inf(1:no);x.sup=x.sup(1:no);

if nargout==3
    % remove slack entries from the constraints
    A.inf=E.inf(:,2:end);A.sup=E.sup(:,2:end);
    ind=1:no;ind2=ind;for i=1:no,ind2=[ind2 i*nx+ind];end
    A.inf=A.inf(:,ind2);A.sup=A.sup(:,ind2);
end
