%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% glqmat2qvec.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function u=glqvec2qmat(G);
%
% converts [1;x]'G[1;x] to u'q(x) where 
% q(x)=[1;x(1);..;x(nx);x(1)x(1);..;x(1)x(nx);..;x(nx)x(1);..;x(nx)x(nx)]
% and x=[x(1);..;x(nx)]
%
% G     (nx+1) x (nx+1) symetric matix representing the quadratic vector
%
% u     1 x (nx+1)nx+1 vector of the quadractic coeffs

function u=glqmat2qvec(G)

% argcheck
if nargin<1, error('glqmat2qvec: Not enough arguments!');end

% init
nx=size(G,1)-1; %number of initial variables

G=G+G'; % sum the transpose
% constant and linear entries
u=sparse([1 2:nx+1], [1 ones(1,nx)], [G(1,1)/2 G(1,2:end)], 1+nx+nx^2, 1, nnz(G)+nx+1);
% quadratic entries
for i=2:nx+1
    G(i,i)=G(i,i)/2; %divide diag
    if i>1, G(i,1:i-1)=0; end; %clear under diag
    u((i-1)*nx+2:i*nx+1)=G(i,2:end);
end