%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glqvec2qmat.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function G=glqvec2qmat(u);
%
% converts u'q(x) to [1;x]'G[1;x] where 
% q(x)=[1;x(1);..;x(nx);x(1)x(1);..;x(1)x(nx);..;x(nx)x(1);..;x(nx)x(nx)]
% and x=[x(1);..;x(nx)].
%
% u         1 x (nx+1)nx+1 vector of the quadractic coeffs
%
% G         (nx+1) x (nx+1) symetric matix representing the quadratic vector

function G=glqvec2qmat(u)

% argcheck
if nargin <1, error('glqvec2qmat: Not enough arguments!');end
% init
nx=((-3+4*(length(u)))^0.5-1)/2; %number of initial variables
if floor(nx) ~=nx
    gldisp('nx',nx);
    error('glqvec2qmat: u has wrong number of entries!');
end
G=sparse([],[],[],nx+1,nx+1,nnz(u)+nnz(u(2:nx+1)));
if size(u,1)<size(u,2)
    u=u';
end
G(1,:)=u(1:nx+1)';
G(2:nx+1,2:nx+1)=reshape(u(nx+2:(nx+1)*nx+1),nx,nx);
% symmetrize
G=(G+G')/2;