%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glqvec2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function out=glqvec2str(u, rel, html);
%
% converts the vector of quadratic coeffs to its string representation
%
%  u           1+nx+nx^2 x 1 vector of quadratic coeffs
%  rel         realtion between the lhs and the rhs of the eq
%              (0 = ,-1 <= ,1 >=) ([] no equation only rhs)
%  html        format as a html text (default=0)
%
%  out        string representation of u

function out=glqvec2str(u, rel, html)

%HTMLCOLOR='#008F8F' ;

% argcheck
if nargin<1, error('glqvec2str: Not enough arguments!');end
if nargin<2, rel=[];end
if nargin<3, html=0;end

% init
nx=((-3+4*(length(u)))^0.5-1)/2; %number of initial variables
if floor(nx) ~=nx
    gldisp('nx',nx);
    error('glqvec2str: u has wrong number of entries!');
end

% create string representation of u
nz=0;out='';ind=[2:length(u) 1];
for i=ind
    c=u(i);
    if i==1 && ~isempty(rel),
        % constant term, rhs of the eq
        if html
            if rel==1, r=' &ge; ';
            elseif rel==-1, r=' &le; ';
            else r=' = ';
            end
            out=[out '<B>' r '</B>'];
        else
            if rel==1, r=' >= ';
            elseif rel==-1, r=' <= ';
            else r=' = ';
            end
            out=[out r];
        end
        out=[out num2str(-c)];
    elseif c~=0,
        % write constants
        if nz~=0 && c>=0,
            out=[out '+'];
        else
            nz=1;
        end
        if c==-1
            out=[out '-'];
        elseif c~=1
            out=[out num2str(c)];
        end
        % write variables
        var='';
        if i==1
        elseif i>1 && i<=nx+1
            % linear term
            if html,
                var=['x<sub><small>' num2str(i-1) '</small></sub>'];
            else
                var=['x_' num2str(i-1)];
            end
        elseif i>nx+1 && i<=length(u)
            % quadratic term
            j=rem(i-1,nx);k=(i-1-j)/nx;
            if j==0, j=nx;k=k-1; end
            if k~=j,
                if html,
                    var=['x<sub><small>' num2str(k)...
                        '</small></sub>x<sub><small>' num2str(j)...
                        '</small></sub>'];
                else
                    var=['x_' num2str(k) 'x_' num2str(j)];
                end
            else 
                if html,
                    var=['x<sub><small>' num2str(k) '</small></sub>'...
                        '<sup><small>2</small></sup>'];
                else
                    var=['x_' num2str(k) '^2'];
                end
            end
        end
        out=[out var];
    end
end