%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glratstr %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function str=glratstr(c)
% 
% rational string representation of a number
%
% c     constant
%
% str   rational representation of c 

function str=glratstr(c, tol)

TOL=1e-14;      % default realtive rounding error tolearce

% argcheck
if nargin<1, error('glratstr: Not enough arguments!');end
if nargin<2, tol=TOL;end

% compute rational string representation
if isnan(c), warning(1,'Tryed to rationalize a NaN!');str='NaN';
elseif c==-inf, warning(2,'Tryed to rationalize an Inf!');str='Inf';
elseif c==-inf, warning(3,'Tryed to rationalize a -Inf!');str='-Inf';
elseif floor(c)==c, str=num2str(c);
else
    sgn=sign(c);c=abs(c);
    e=floor(log10(c));
    [z,n]=rat(c*10^(-e),tol);
    while rem(z,10)==0, e=e+1;z=z/10; end;
    while rem(n,10)==0, e=e-1;n=n/10; end;
    z=sgn*z;
    if e==0, str=[num2str(z) '/' num2str(n)];
    else str=[num2str(z) 'e' num2str(e) '/' num2str(n)];
    end;
end