%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glscell2struct %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% var=glscell2struct(scell, var)
%
% Insert the fields into a structure from a string cell produced by the
% glstruct2str method.
%
% strc  the input string cell produced by glstruct2str
% var   the structure in which the fields are inserted (or overwritten)
%
% var   the output structure

function var=glscell2struct(scell, var)
line=1;var=struct;
while line <= length(scell)
    tx=scell{line};
    if any(tx=='{')
        %cell array
        while line <= length(scell)
            %merge whole cell in one string
            bn=length(find(tx=='[')); % number of '['
            en=length(find(tx==']')); % number of ']'
            if bn ~= en, line=line+1;tx=[tx scell{line}];
            else break;
            end
        end
        if bn ~= en
            gldisp('tx',tx);
            error('glscell2struct: cell in wrong format!');
        end
    elseif length(find(tx=='"'))==1
        % merge multiline text
        line=line+1;fin=0;
        while line <= length(scell)
            lin=scell{line};tx=[tx lin];
            if any(lin=='"'), fin=1; break;
            else line=line+1;
            end
        end
        if ~fin
            gldisp('tx',tx);
            error('glscell2struct: cell in wrong format!');
        end
    end
    var=str2var(tx,var);
    line=line+1;
end
    

function var=str2var(str, var)
if nargin ==1, var=[]; end
if isempty(str) || str(1)=='%' || str(2)=='%',
    %empty line or comment line
    %find last line comment number
    fields = fieldnames(var);cn=0;
    for i=1:length(fields)
        ind=findstr(fields{i},'comment');
        if ~isempty(ind) && ~isempty(str2num(fields{i}(ind+7:end)))
            cn=max(cn,str2num(fields{i}(ind+7:end)));
        end
    end
    %set comment variable : comment(number)
    var.(['comment' num2str(cn)])=str;
else
    %no comment :)
    %parse list
    lind1=find(str==';');
    if isempty(lind1), lind=[0 length(str)];
    else
        lind1=[0 lind1];lind=0;j=1;
        for i=1:length(lind1)-1
            tx=str(lind(j)+1:lind1(i+1)-1);
            no=length(find(tx=='{'))+length(find(tx=='['));
            nc=length(find(tx=='}'))+length(find(tx==']'));
            if no==nc, lind=[lind lind1(i+1)]; j=j+1; end
        end
    end
    for i=1:length(lind)-1
        tx=str(lind(i)+1:lind(i+1));
        ind=find(tx=='=');
        if ~isempty(ind) && ~any(tx(1:ind(1)-1)=='"')
            %named variable
            ind2=find(tx==';');
            if isempty(ind2)
                error(['str2var: cell structure in wrong format, ; '...
                    'is missing in the line ' tx '!']);
            end
            ind2=ind2(end);
            str1=tx(1:ind(1)-1);str1=str1(str1~=' ');       %varname
            str2=tx(ind(1)+1:ind2(end)-1);                  %varcontent
            comm=tx(ind2+1:end);                            %comment
        else
            %nonname varaibale
            str1='noname';str2=str;comm=[];
        end
        if any(str1=='.')
            %struct
            ind3=find(str1=='.');name=str1(1:ind3(1)-1);
            v=str2var([str1(ind3(1)+1:end) '=' str2 ';']);
            fields = fieldnames(v);field=fields{1};
            if ~isfield(var,name), var.(name)=[]; end
            if isfield(var.(name), field), 
                % the field already exists merge with the old one
                fs = fieldnames(v.(field));
                var.(name).(field).(fs{1})=v.(field).(fs{1});
            else
                % new field 
                var.(name).(field)=v.(field);
            end
        elseif any(str2=='{')
            %cell array
            bind=find(str2=='{')+1;eind=find(str2=='}')-1; 
            if length(bind) ~= length(eind)
                gldisp('str2',str2);
                error('str2var: cell structure in wrong format!');
            end
            if any(str2==';'), var.(str1)=cell(length(bind),1);
            else var.(str1)=cell(1,length(bind)); end
            for j=1:length(bind)
                v=str2var(str2(bind(j):eind(j)));
                fields = fieldnames(v);
                var.(str1){j}=v.(fields{1});
            end
        elseif ~isempty(str2num(str2))
            %numerical data
            var.(str1)=str2num(str2);
        else
            ind3=find(str2=='"');
            if length(ind3)==2
                %string
                var.(str1)=str2(ind3(1)+1:ind3(2)-1);
            else
                %wrong type
                gldisp('str',str,'str2',str2);
                error('str2var: wrong format!');
            end
        end
        if ~isempty(comm)
            %comment found
            var.([str1 '_comment'])=comm;
        end;
    end
end