%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glstr2scell.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function scell=glstr2scell(str, del)
%
% converts a multiline string (delimeter del) to string cell array
%
% str   input string 
% del   line delimeter (='\n') if none supplied 
%       the delim char(10) is often used in unix outputs
% 
% scell string cell array created from the string

function scell=glstr2scell(str, del)

% argcheck
if nargin<1, error('glstr2scell: Not enough arguments!');end
if nargin<2, del='\n';end
% delim length
if ischar(del), del=char(del); end;
dl=length(del);
% apply del at the end of the string if none present
if ~strcmp(str(end-dl+1:end), del), str=[str del];end
% find indices of del
ind = [1-dl strfind(str, del)];n=length(ind)-1;
% convert str to scell
scell=cell(n,1);
for i=1:n, 
    scell{i}=str(ind(i)+dl:ind(i+1)-1);
end