%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glstruct2scell %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function strc=glstruct2scell(var, varprefix)
%
% Convert the fields of the given structure in a sofisticated output 
% string cell. Constants, vectors, strings, and cell arrays, structure 
% and comments ((varname)_comment = comment after the variable definition
% comment... comment line) are allowed inside the structure.
%
% var   the input structure
%
% strc  output string cell

function strc=glstruct2scell(var, varprefix)
names = fieldnames(var);strc=cell(0);
for i=1:length(names),
    if strfind(names{i},'_comment'), continue;
    elseif strfind(names{i},'comment')
        strc=[strc ; {var.(names{i})}];  %comment row
        continue;
    end
    dat=var.(names{i});
    if nargin ==2
        if ~isempty(varprefix), txt={[varprefix '.' names{i} ' = ']};
        else txt={''};
        end
    else txt={[names{i} ' = ']};
    end
    if isempty(dat)
        %empty txt
        txt{1}=[txt{1} '"";'];
    elseif isnumeric(dat)
        %numeric txt
        txt{1}=[txt{1} mat2str(dat) ';'];
    elseif islogical(dat)
        %logical txt
        txt{1}=[txt{1} mat2str(dat) ';'];
    elseif iscell(dat)
        %cell txt
        spac=' ';sl=1;
        if size(dat,1)>size(dat,2), delim=';';
        else delim=',';
        end
        txt{1}=[txt{1} '['];
        for j=1:length(dat)
           nl=min(3,length(names{i}));
           v=[];v.(names{i}(1:nl))=dat{j};
           tx=glstruct2scell(v, []);
           if length(tx)==1
               txt{end}=[txt{end} '{' tx{1}(1:end-1) '}' delim];
           else
               txt{end}=[txt{end} '{'];
               for n=1:length(tx),
                   txt=[txt ; {[spac spac spac tx{n}]}];
               end;
               txt=[txt; {[spac spac '}' delim]}];
               sl=length(spac)+1;
           end
        end
        txt{end}=[txt{end}(sl:end-1) '];'];
    elseif isstruct(dat)
        %structure
        if nargin ==2 && ~isempty(varprefix), pref=[varprefix '.'];
        else pref=[];
        end
        txt=glstruct2scell(dat, [pref names{i}]);
    else
        %string txt
        txt{1}=[txt{1} '"' dat '";'];
    end
    %variable comment
    if isfield(var,[names{i} '_comment'])
        comm=var.([names{i} '_comment']);
    else comm=[]; %no comment :)
    end;
    txt{end}=[txt{end} comm];
    strc=[strc ; txt];
end