%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gltask2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function str=gltask2str(task, def)
%
% Convert a task to its string representation. The fields 
%
% task          input task with different fields
% def           default flag 0=all fields of the task are written ou
%               1=only the fields with value different from the value
%               set in the config are written.
%           
% str           output string

function str=gltask2str(task, def)

global glconf_default;    % global default configuration variable

% argcechk
if nargin<1, error('gltask2str: Not enough arguments!');end
if nargin<2, def=0;end

% init
tnam=task.name;str=tnam;
if isfield(task, 'argin')
    % additional argin fields are present
    tfield=task.argin;
    if isfield(tfield,'method'),
        method=tfield.method;method(method=='_')=' ';
        str=[str ' (' method ')'];
        tfield=rmfield(tfield, 'method');
    end
    fnams=fieldnames(tfield);n=length(fnams);
    if length(tnam)>6 && strcmp(tnam(1:6),'Begin '), tpre=lower(tnam(7:10));
    else tpre=lower(tnam(1:4));
    end
    if isfield(glconf_default, tpre), cfield=glconf_default.(tpre);
    else cfield=[];
    end

    % create arg string
    txt=[];
    for i=1:n
        fn=fnams{i};fv=tfield.(fn);
        pflag=1;    % printing flag
        if def && isfield(cfield,fn)
            % check default is on and the field is in the config
            cv=cfield.(fn);
            if (ischar(fv) && strcmp(fv,cv)) || (isnumeric(fv) && ...
                    abs(fv-cv)<eps) || (islogical(fv) && fv==cv)
                % field has the same numeric value as in the config or the
                % field has the same string value as in the conf, do not print
                pflag=0;
            end
        end
        if pflag,
            % print the field
            if ischar(fv), 
                txt=[txt fn '=' fv ', '];
            elseif isnumeric(fv), 
                if fv~=0, txt=[txt fn '=' num2str(fv) ', '];end
            elseif islogical(fv), 
                if fv, txt=[txt fn ', '];
                elseif def, txt=[txt 'no' fn ', '];
                end
            else error('gltask2str: Unknown data type found!');
            end
        end
    end

    % add arg string
    if ~isempty(txt), str=[str ' - ' txt(1:end-2)];
    elseif n>0 && def, str=[str ' - default'];
    end
end