%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glvar2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function str=glvar2str(ind, mode, origvarnams)
%
% Convert a variable with the given index to normal, html or ampl format.
% If origvarnams is not empty and the index is inside of this cell the 
% original variable name is returned else a string like x_3 
%
% ind          index of the variable
% mode         text output mode: 0=normal text, 1=html, 2=ampl format
% origvarnams  no x 1 cell of originalvariable names or empty
%
% str          string representation of the variable

function str=glvar2str(ind, mode, origvarnams)

% argcheck
if nargin <1, error('glvar2str: Not enough arguments!');end
if nargin <2, mode=0;end
if nargin <3, origvarnams=[];end

if ~isempty(origvarnams) && length(origvarnams)>=ind
    %get the original variable name
    str=origvarnams{ind};
else
    %generate variable name
    if mode==0, str=['x_' num2str(ind)];
    elseif mode==1, str=['x<sub><small>' num2str(ind) '</sub></small>'];
    else str=['x' num2str(ind)];
    end
end
        