%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glconc.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function x=glconc(x, y, ind)
% 
% concatenate box x with the box y
%
% x, y      boxes of the same dimension
%   .inf    nx x 1 lower bounds
%   .sup    nx x 1 upper bounds
% ind       optional index set of components which should be concatenated
%
% x         result of the concatenation x \conc y, empty or with
%   .inf    nx x 1 lower bounds
%   .sup    nx x 1 upper bounds

function x=glconc(x, y, ind)

% argcheck
if nargin <2, error('glconc: Not enough arguments!');end
if nargin <3, ind=1:length(x.inf);end
if length(x.inf)~=length(y.inf), error('glconc: Dimension mismatch!');end

% convert intvals to structs
if isintval(x), xx.inf=x.inf; xx.sup=x.sup; x=xx; end
if isintval(y), yy.inf=y.inf; yy.sup=y.sup; y=yy; end

% concatenate box x with the box y
x.inf(ind)=max(x.inf(ind),y.inf(ind));
x.sup(ind)=min(x.sup(ind),y.sup(ind));

% check result
if any(x.inf>x.sup)
    % problem is infeasible
    x=[];
end
