%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glimult.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function C=glimult(A, B)
% 
% Multiply two matrices. This is now done correctly for the case if
% some interval multiplication [-inf inf]*0 or [-inf inf]*[0 0] happens.
%
% A         scalar or an n x t real or interval matrix
% B         scalar or an t x m real or interval matrix
%
% C         real or interval matrix or struct C=A*B

function C=glimult(A, B)

% argcheck
if nargin <2, error('glimult: Not enough arguments!');end

%dimension check
[n,t]=size(A);[t2,m]=size(B);
if ~(n==1 && t==1) && ~(t2==1 && m==1)
    %matrix times matix
    if t~=t2, error('glimult: Matrix dimensions mismatch!');end
    C=zeros(n,m);
    if isintval(A) || isintval(B), C=intval(C);end
    ind1=(A~=0);ind2=(B'~=0);
    for i=1:n
        for j=1:m
            ind=find(ind1(i,:) & ind2(j,:));
            C(i,j)=glisprod(A(i,ind),B(ind,j));
        end
    end
elseif (n==1 && t==1) && (t2==1 && m==1)
    %both A and B is a scalar
    if A==0 || B==0, 
        if isintval(A) || isintval(B), C=infsup(0,0);
        elseif isinf(A) || isinf(B), C=nan;
        else C=0;
        end
    else C=A*B;
    end
elseif n==1 && t==1
    %A is a scalar
    C=zeros(t2,m);
    if isintval(A) || isintval(B), C=intval(C);end
    if A~=0,ind=B~=0;C(ind)=A*(B(ind));end
else
    %B is a scalar
    C=zeros(n,t);
    if isintval(A) || isintval(B), C=intval(C);end
    if B~=0,ind=A~=0;C(ind)=B*A(ind);end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glisprod.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function c=glisprod(a, b)
% 
% Scalar product of two vectors. This is also correct for cases
% like [-1 -1 1]*infsup([-inf; -inf; 0],[-2; 5; inf]) which gives 
% [nan, inf] if using the intlab multiplication.
%
% a         n-dim real or interval vector
% b         n-dim real or interval vector
%
% c         real or interval contant c=<a,b>

function c=glisprod(a, b)
% argcheck
if nargin <2, error('glisprod: Not enough arguments!');end

%dimension check
n=length(a);
if length(b)~=n, error('glimult: Vector dimensions mismatch!');end
c=intval(0);
if isintval(a) || isintval(b), c=intval(c);end
for i=1:n, c=c+a(i)*b(i); end
