%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmid.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function z=glmid(x, ind)
% 
% Compute the midpoint of the box x.
%
% x         box
%   .inf    nx x 1 lower bounds
%   .sup    nx x 1 upper bounds
% ind       index set of the allowed components of x (optional)
%
% z         nx x 1 or length(ind) x 1 midpoint vector

function z=glmid(x, ind)

% argcheck
if nargin <1, error('glmid: Not enough arguments!');end
if nargin <2, ind=1:length(x.inf);end

% compute the naive midpoint of the box
z=(x.inf(ind)+x.sup(ind))/2; 

% set midpoint of the infinite components
binf=isnan(z);ninf=(z==-inf);pinf=(z==inf); %infinite cases indices
z(binf)=0;z(ninf)=x.sup(ind(ninf));z(pinf)=x.inf(ind(pinf));