%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glscale.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [A, F, x]=glscale(A, F, E, x, xs, cs, prt)
% 
% scale the given problem, Aq(x) \in F,  set x=xs.*x' obtain
% S=cs*1, U=q(xs)*1, then A'=SAU, with , F'=cs.*A, and x'=x/xs.
% Scale back with x=x'*xs.
%
% A             nc x np matrix of inequality constraint coeffs
%   .inf,.sup   bounds for coefficients
% F             nx x 1 general constraint bounds
%   .inf.sup 
% x             nx x 1 bound constraints
%   .inf.sup 
% E             np x nx matrix representing the monomial vector 
%               q(x)_k=\sum_{j=1...nx} x_j^(E_{k,j})
% xs            nx x 1 variable scaling vector
% cs            nc x 1 constraints scaling vector
%
% A             new nc x np matrix of inequality constraint coeffs
%   .inf,.sup   bounds for coefficients
% F             new nx x 1 general constraint bounds
%   .inf.sup 
% x             new nx x 1 bound constraints
%   .inf.sup 

function [A, F, x]=glscale(A, F, E, x, xs, cs, prt)

% argcheck
if nargin <6, error('glscaling: Not enough arguments!');end
if nargin <7, prt=0;end
nc=size(A.inf,1);
np=size(E,1);
qx=zeros(np,1);
for k=1:np,
    qx(k)=prod(xs'.^E(k,:));
end
setround(1);
S=cs*ones(1,np);U=ones(nc,1)*qx';
A.inf=-(S.*(-A.inf).*U);A.sup=S.*A.sup.*U;
x.inf=-((-x.inf)./xs);x.sup=x.sup./xs;
F.inf=-((-F.inf).*cs);F.sup=F.sup.*cs;
if prt, gldisp('Ainf',A.inf,'F',[F.inf F.sup],'x',[x.inf x.sup]); end

