%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glchkconf.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function miss=glchkconf(cfilen)
%
% Check all fields of the given configuration by compairing it to the
% sample. The missing fields are returned.
%
% cfilen   name of the configuration file (optional)
%
% miss     1 x nm cell of the missing nm field names or -1 if the config
%          file is missing.

function miss=glchkconf(cfilen)

% argcheck
if nargin<1,
    if isunix, cfilen='Gloptlab_unix.cfg';  % default unix config file
    else cfilen='Gloptlab_win.cfg';         % default windows config file
    end
end

% load sample
if isunix, sconf=glload('Gloptlab_unix_sample.cfg');% unix sample config
else sconf=glload('Gloptlab_win_sample.cfg');       % windows sample config
end

% load the config and compare it to the sample
if exist(cfilen,'file')==2
    conf=glload(cfilen);
    fnames = fieldnames(sconf);
    miss=cell(0);
    for i=1:length(fnames)
        fn=fnames{i};
        if ~isfield(conf, fn)
            % missing field
            miss=[miss {fn}];
        elseif isstruct(sconf.(fn))
            % scan subfields
            field=sconf.(fn);
            fnames2 = fieldnames(field);
            for j=1:length(fnames2)
                fn2=fnames2{j};
                if ~isfield(conf.(fn), fn2)
                    % missing subfield
                    miss=[miss {[fn '.' fn2]}];
                end
            end
        end
    end
else miss=-1;
end