%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% glcreatetimer.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function gltimer=glcreatetimer(timeout, outptime, prt, log, savetime, 
% logdir, autosavedir)
%
% Create a timer structure.
%
% gltimer           timer structure, starting/reseting timer is
%                   possible by using the 'tic' command, all times
%                   are given in seconds
%        .status    0 = off, 1 = on, 2 = paused 
%        .timeout   seconds until timeout is reaced
%        .outptime  seconds between two text and/or log outputs
%        .lastoutp  time of the last text and/or log output
%        .prt       =0 no outp, =1 brief output 2= full output, 3= both
%        .log       =1 indicates log saving enabled
%        .savetime  seconds between two automatic saves
%        .lastsave  time of the last save
%        .savedir   the directory to store the saves, if empty use config
%        .logdir    the directory to store the logs, if empty use config
%        .ellaped   the total time ellapsed = .ellapsed+toc

function gltimer=glcreatetimer(timeout, outptime, prt, log, savetime, ...
    logdir, autosavedir)

%argcheck
if nargin < 1,error('glcreatetimer: Not enough arguments!'); end

gltimer=[];
gltimer.status      = 0;
gltimer.timeout     = timeout;
gltimer.outptime    = outptime;
gltimer.prt         = prt;
gltimer.log         = log;
gltimer.savetime    = savetime;
gltimer.logdir      = logdir;
gltimer.autosavedir = autosavedir;
gltimer.ellapsed     = 0;

