%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gldispquad.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function gldispquad(a,b,c,k,N);
%
% displays the text representation of the quadratic functions 
% q_i(x)=a(i)x(k(i))^2+b(i)x(k(i))>=c(i)
% if N is given displays Nq(x)>=c,  q_i(x)=a(i)x(k(i))^2+b(i)x(k(i))
%
%   a,b,c,k         1 x n vectors (normal or interval)
%   N               nc x n dimensional (0,1) coefficient matrix
%   rev             if != 0 reverse the >= to <=

function gldispquad(a,b,c,k,N,rev)

global glconf;  %global configuration variable
PREC=1e-6;      %percision displaying 1 instead of x in [1-PREC,1+PREC]

%argcheck
if nargin<4, error('gldispquad: Not enough arguments!');end
if nargin<5 || isempty(N), N=eye(length(a),length(a));end
if nargin<6, rev=0;end

%set constants from config
if isfield(glconf,'tiny'),PREC=glconf.tiny;end

%display polynomials
for i=1:size(N,1)
    out=[];
    inds=find(N(i,:) ~= 0);
    if isempty(inds)
        continue;
    end
    %non constant parts
    for j=inds
        if  b(j) ~= 0   %linear part
            if isintval(b)  %interval coef
                if j~=inds(1),out=[out '+'];end
                out=[out '[' num2str(b(j).inf) ',' num2str(b(j).sup) ']x_' num2str(k(j))];
            else        %non interval coef
                if -1-PREC<b(j) && b(j)<-1+PREC,out=[out '-'];
                elseif 1-PREC<b(j) && b(j)<1+PREC
                    if j~=inds(1),out=[out '+'];end
                else
                    if b(j)>0 && j~=inds(1), out=[out '+'];end
                    out=[out num2str(b(j))];
                end
                out=[out 'x_' num2str(k(j))];
            end
        end
        if a(j) ~= 0    %quadratic part
            if isintval(a)  %interval coef
                if j~=inds(1),out=[out '+'];end
                out=[out '[' num2str(a(j).inf) ',' num2str(a(j).sup) ']x_' num2str(k(j)) '^2'];
            else            %non interval coef
                if -1-PREC<a(j) && a(j)<-1+PREC,
                    out=[out '-'];
                elseif 1-PREC<a(j) && a(j)<1+PREC 
                    if b(j) ~= 0 || j~=inds(1), out=[out '+'];end
                else
                    if a(j)>0, out=[out '+'];end
                    out=[out num2str(a(j))];
                end
                out=[out 'x_' num2str(k(j)) '^2'];
            end
        end
        if a(j) == 0 && b(j) == 0 && length(inds)==1
            out=[out '0'];
        end
    end
    %constant part
    if rev, op='<=';
    else op='>=';
    end
        
    if isintval(c)  %interval constant
        out=[out ' ' op ' [' num2str(c(i).sup) ',' num2str(c(i).inf) ']'];
    else            %non interval constant
        out=[out ' ' op ' ' num2str(c(i))];
    end
    disp(out);
end
