%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glfindfun.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function postfixes=glfindfun(prefix, topdir)
% find all possible variants of the matlab functions with the given
% prefix in the file system below the supplied topdir
% and return all the corresponding postfixes.
%
% prefix        string of the prefix of the function e.g. 'glsplit_'
% topdir        full path of the directory below the file system should
%               be searched for the prefix
%
% postfixes     string cell of the postfixes of all functions in the matlab
%               path which have the supplied prefix. e.g. 'maxvariation' if
%               the function glsplit_maxvariation.m exists below the topdir

function postfixes=glfindfun(prefix, topdir)

% argcheck
if nargin<2, error('glfindfun: Not enough arguments!');end

%init
if (topdir(end)~='/') && (topdir(end)~='\'), topdir=[topdir filesep];end
lp=length(prefix);

% search the direcctories below the topdir for files which begin with the
% supplied prefix 
dirs={topdir};      %directories to search
postfixes=cell(0);  %found postfixes
while ~isempty(dirs)
    % get directory listing of the current dir and remove the current
    % dir from the directory seach list
    cdir=dirs{1}; dirs(1)=[];
    files=dir(cdir);
    % separate subdirectories and the files:
    % --add subdirs to the search list but do not add the
    %   the '.' and '..' directories
    % --check the files if they have the required prefix and if yes
    %   save their postfix in the postfixes string cell
    for i=1:length(files)
        nam=files(i).name;
        if files(i).isdir
            % the dir entry is a directory
            if ~strcmp(nam,'.') && ~strcmp(nam,'..')
                %add it to the directory seach list
                dirs=[dirs ; {[cdir nam filesep]}];
            end
        elseif length(nam)>=lp && strcmp(nam(1:lp),prefix)
            % the entry is a file with the given prefix, save postfix if
            % it is an .m file
            [empty,nam,ext]=fileparts(nam);
            post=nam(lp+1:end);
            if strcmp(ext, '.m') && ...
              ~strcmp(post, 'gui') && ~strcmp(post, 'sample') 
                postfixes=[postfixes ; {post}];
            end
        end
            
    end
end
