%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glinitconf.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function glinitconf(cfilen, extrachecks)
%
% loads the configuration given by the supplied filename, check and creates
% all directories, checks the coconut path, initializes intlab. 
%
% cfilen        name of the configuration file
% extrachecks   =1(default): check coconut, testenvironment and viewers
% prt           text output flag (default: turned on)

function glinitconf(cfilen, extrachecks, prt)

global glconf;           % global configuration structure
global glconf_default;   % default configuration
global glconfstat;       % status of the loaded configuration
    
%init global vars
if ~isfield(glconfstat, 'intlab_running'), glconfstat.intlab_running=0; end
if ~isfield(glconfstat, 'lpsolve_ok'),     glconfstat.lpsolve_ok=0; end
if ~isfield(glconfstat, 'sedumi_ok'),      glconfstat.sedumi_ok=0; end
if ~isfield(glconfstat, 'sdpt3_ok'),       glconfstat.sdpt3_ok=0; end
if ~isfield(glconfstat, 'coconut_ok'),     glconfstat.coconut_ok=0; end
if ~isfield(glconfstat, 'htmlview_ok'),    glconfstat.htmlview_ok=0; end
if ~isfield(glconfstat, 'psview_ok'),      glconfstat.psview_ok=0; end
if ~isfield(glconfstat, 'pdfview_ok'),     glconfstat.pdfview_ok=0; end

%argcheck
if nargin<1, 
    if isunix, cfilen='Gloptlab_unix.cfg';  %default unix config file
    else cfilen='Gloptlab_win.cfg';         %default windows config file
    end
end
if nargin<2, extrachecks=0;end
if nargin<3, prt=1;end

% CHECK CONFIG
if prt,disp(' ');disp('*CHECKING THE SELECTED CONFIG FILE*');end
basedir=[fileparts(which('Gloptlab.m')) filesep];
if exist(cfilen,'file')~=2
    % if no default config exits, use the sample config and the Globtlab.ini
    % file (if exists) to create a new one.
    if prt,disp(['Config file ' cfilen ' was not found!']); end
    if isunix, cfilen='Gloptlab_unix.cfg';  %default unix config file
    else cfilen='Gloptlab_win.cfg';         %default windows config file
    end
    f1=[basedir cfilen];f2=[basedir cfilen(1:end-4) '_sample.cfg'];
    [s,mess]=copyfile(f2, f1);
    if s, 
        if prt,
            disp(['Config file ' f2 ' copied to ']);
            disp(['   ' f1 '!']);
            disp(['Using the config file ' f1 '!']);
        end
        inifile=[basedir 'Gloptlab.ini']; 
        if exist(inifile,'file')==2
            % copy the contents of the ini file into the created
            % config file
            fid=fopen(inifile);
            vars=cell(0);conts=cell(0);
            while 1
                tline=fgetl(fid);
                if ~ischar(tline), break, end
                ind=find(tline=='=');
                if ~isempty(ind) && length(ind)==1
                    var=tline(1:ind-1);
                    while 1,
                        % remove spaces from the end
                        if var(end)==' ', var=var(1:end-1);
                        else break;
                        end
                    end
                    cont=tline(ind+1:end);
                    ind=find(cont=='"');
                    if ~isempty(ind) && length(ind)==2
                        cont=cont(ind(1)+1:ind(2)-1);
                        vars=[vars;{var}];
                        conts=[conts;{cont}];
                    else continue;
                    end   
                end 
            end
            fclose(fid);
            if ~isempty(vars)
                % read the new configuration file
                vn=length(vars);
                fid=fopen(f1);
                txt=cell(0);
                while 1
                    % read the config file
                    tline = fgetl(fid);
                    if ~ischar(tline), break, end
                    txt=[txt; {tline}];
                end
                fclose(fid);
                % replace the contents of the variables in the config file
                % with the content of the same variables from the .ini file
                for i=1:length(txt)
                    tline=txt{i};
                    ind=find(tline=='=');
                    if ~isempty(ind) && length(ind)==1
                        var=tline(1:ind-1);
                        while 1,
                            % remove spaces from the end
                            if var(end)==' ', var=var(1:end-1);
                            else break;
                            end
                        end
                        for j=1:vn
                            if strcmp(var, vars{j})
                                % replace the old content of the variable
                                % witht the content from the .ini file
                                ind=find(tline=='"');
                                if ~isempty(ind) && length(ind)==2
                                    txt{i}=[tline(1:ind(1)) conts{j} ...
                                        tline(ind(2):end)];
                                    break;
                                end
                            end
                        end
                    end
                end
                % save the modified config file
                fid=fopen(f1,'w');
                for i=1:length(txt)
                    fprintf(fid,'%c',txt{i});
                    fprintf(fid,'\n');
                end
                fclose(fid);
            end
        end
    else
        error(['Error copying config file ' f2 ' to ' f1 ', message:'
            mess '!']);
    end
elseif prt,disp(['Config file ' cfilen ' has been found!']);
end

% LOAD CONFIG
glconf=glload(cfilen);

% init linear and conic solver selection
glconf.lsolvers=cell(0);
glconf.csolvers=cell(0);

% CHECK AND CREATE DIRECTORIES 
if prt,
    disp(' ');disp('*CHECKING THE PREDEFINED DIRECTORIES*');
    disp(['Gloptlab directory: ' basedir]);
end
glconf.gloptdir=basedir;
% working dir
if ~isfield(glconf, 'workingdir') || isempty(glconf.workingdir)
    glconf.workingdir=[basedir 'Testproblems' filesep];
elseif ~isempty(glconf.workingdir) && (glconf.workingdir(end)~=filesep)
    glconf.workingdir=[glconf.workingdir filesep];
end
if exist(glconf.workingdir,'dir')~=7, 
    mkdir(glconf.workingdir);
    if prt,disp(['Working directory ' glconf.workingdir ' created...']);end
elseif prt,disp(['Working directory: ' glconf.workingdir]);
end
% strategy dir
if ~isfield(glconf, 'stratdir') || isempty(glconf.stratdir)
    glconf.stratdir=[basedir 'Strategies' filesep];
elseif ~isempty(glconf.stratdir) && glconf.stratdir(end)~=filesep
    glconf.stratdir=[glconf.stratdir filesep];
end
if exist(glconf.stratdir,'dir')~=7, 
    mkdir(glconf.stratdir);
    if prt,disp(['Strategies directory ' glconf.stratdir ' created...']);end
elseif prt,disp(['Strategies directory: ' glconf.stratdir]);
end
% solution dir
if ~isfield(glconf, 'soldir') || isempty(glconf.soldir)
    glconf.soldir=[basedir 'Output' filesep 'Solutions' filesep];
elseif ~isempty(glconf.soldir) && glconf.soldir(end)~=filesep
    glconf.soldir=[glconf.soldir filesep];
end
if exist(glconf.soldir,'dir')~=7, 
    mkdir(glconf.soldir); 
    if prt,disp(['Solutions directory ' glconf.soldir ' created...']);end
elseif prt,disp(['Solutions directory: ' glconf.soldir]);
end
% autosave dir
if ~isfield(glconf, 'autosavedir') || isempty(glconf.autosavedir)
    glconf.autosavedir=[basedir 'Output' filesep 'Autosaves' filesep];
elseif ~isempty(glconf.autosavedir) && glconf.autosavedir(end)~=filesep
    glconf.autosavedir=[glconf.autosavedir filesep];
end
if exist(glconf.autosavedir,'dir')~=7, 
    mkdir(glconf.autosavedir); 
    if prt,disp(['Autosaves directory ' glconf.autosavedir ' created...']);end
elseif prt,disp(['Autosaves directory: ' glconf.autosavedir]);
end
% log dir
if ~isfield(glconf, 'logdir') || isempty(glconf.logdir)
    glconf.logdir=[basedir 'Output' filesep 'Logs' filesep];
elseif ~isempty(glconf.logdir) && glconf.logdir(end)~=filesep
    glconf.logdir=[glconf.logdir filesep];
end
if exist(glconf.logdir,'dir')~=7, 
    mkdir(glconf.logdir);
    if prt,disp(['Logs directory ' glconf.logdir ' created...']);end
elseif prt,disp(['Logs directory: ' glconf.logdir]);
end
% proof dir
if ~isfield(glconf, 'proofdir') || isempty(glconf.proofdir)
    glconf.proofdir=[basedir 'Output' filesep 'Proofs' filesep];
elseif ~isempty(glconf.proofdir) && glconf.proofdir(end)~=filesep
    glconf.proofdir=[glconf.proofdir filesep];
end
if exist(glconf.proofdir,'dir')~=7, 
    mkdir(glconf.proofdir); 
    if prt,disp(['Proofs directory ' glconf.proofdir ' created...']);end
elseif prt,disp(['Proofs directory: ' glconf.proofdir]);
end
% statistic file and dir
if ~isfield(glconf,'statfile'),
    glconf.statfile=[basedir 'Output' filesep 'Statistics' filesep ...
        'statistic.gdb'];
end
statdir=[fileparts(glconf.statfile) filesep ];
if exist(statdir,'dir')~=7, 
    mkdir(statdir); 
    if prt,disp(['Statistics directory ' statdir ' created...']);end
elseif prt,disp(['Statistics directory: ' statdir]);
end

%START INTLAB
if prt,disp(' ');disp('*STARTING INTLAB*');end
if ~isempty(glconf.intlabpath) && glconf.intlabpath(end)~=filesep
    glconf.intlabpath=[glconf.intlabpath filesep];
end
if (isempty(strfind(path,'Intlab')) || glconfstat.intlab_running==0)
    if isfield(glconf, 'intlabpath') && exist(glconf.intlabpath,'dir')==7
        % start intlab
        gltoolinit('Intlab',glconf.intlabpath);
        glconfstat.intlab_running=1;
    else
        if prt, 
            disp(['Intlab is needed to use Gloptlab, check '...
            'the entry intlabpath in Glconfig.cfg !']);
        end
        glconfstat.intlab_running=0;
    end
else
    glconfstat.intlab_running=1;
    if prt,disp('Intlab is already running...'); end
end

%CHECK LOCAL SOLVERS
if prt,disp(' ');disp('*CHECKING THE LOCAL SOLVERS*');end
glconf.locsolvers=[{'lbfgs-b'};{'bfgs'};{'grad proj'}];
if exist('fmincon','file')==2,
    if prt,disp('Fmincon sucessfully located!');end
    glconf.locsolvers=[glconf.locsolvers;{'fmincon'}];
    glconfstat.fmincon_ok=1;
else
    if prt,
        disp(['"fmincon" cannot be located, all local solver '...
            'selections using fmincon are unavailable.']);
        disp(['Install the Optimization Toolbox in order to enable '...
            'this option.']);
    end
    glconfstat.fmincon_ok=0;
end

%CHECK LINEAR SOLVERS
if prt,disp(' ');disp('*CHECKING THE LINEAR SOLVERS*'); end
if ~isempty(glconf.lpsolvepath) && glconf.lpsolvepath(end)~=filesep
    glconf.lpsolvepath=[glconf.lpsolvepath filesep];
end
if ~isfield(glconf, 'lpsolvepath') || exist(glconf.lpsolvepath,'dir')~=7 ...
        || exist([glconf.lpsolvepath 'mxlpsolve.m'],'file')~=2        
    if prt, 
        disp(['LpSolve cannot be located, all linear programms usig '...
          'LpSolve are unusable!']);
    end
    glconfstat.lpsolve_ok=0;
else
    if(ispc)
        %check the windows path to LPsolve
        [dummy, wpath]=glsystem('path');
        if(isempty(strfind(wpath,glconf.lpsolvepath(1:end-1))))
            if prt,
                disp( '*****************************************************');
                disp( '* The path to the dlls used by LpSolve has to be set,');
                disp(['* by adding "' glconf.lpsolvepath ';"'])
                disp( '* to the windows environment variable "path", ')
                disp( '* in Start->Control Panel->System->Properties');
                disp( '* ->Environment Variables! Then restart Matlab!');
                disp( '*****************************************************');
                disp( 'Linear tasks where LpSolve has been selected are unusable!');
            end
            glconfstat.lpsolve_ok=0;
        else
            if prt,disp('LpSolve sucessfully located!');end
            glconfstat.lpsolve_ok=1;
            glconf.lsolvers=[glconf.lsolvers; {'LpSolve'}];
        end
    else
        if exist([glconf.lpsolvepath 'mxlpsolve.mexglx'],'file')~=2 
            glconfstat.lpsolve_ok=0;
            if prt,
                disp( '*****************************************************');
                disp( '* The LpSolve matlab interface mex file ');
                disp(['* "' glconf.lpsolvepath 'mxlpsolve.mexglx", ']);
                disp( '* has not been found. If you have downloaded');
                disp( '* the full version of GloptLab, please run "Makefile.m"');
                disp( '* in above directory to compile.');
                disp( '*****************************************************');
                disp( 'Linear tasks where LpSolve has been selected are unusable!');
            end
        else
            %[dummy,ldpath] = glsystem('echo $LD_LIBRARY_PATH');
            % should check if lpsolve libraries (liblpsolve55.a/.so) are in any of the
            % ld_library_path directories, but in the hand-installed case
            % its hard to tell what architecture they are for, and in the
            % packaged case they are present anyway.
            
            if prt,disp('LpSolve sucessfully located!');end
            glconfstat.lpsolve_ok=1;
            glconf.lsolvers=[glconf.lsolvers; {'LpSolve'}];
        end
    end
end

%CHECK CONIC SOLVERS
if prt,disp(' ');disp('*CHECKING THE CONIC SOLVERS*'); end
if ~isempty(glconf.sedumipath) && glconf.sedumipath(end)~=filesep
    glconf.sedumipath=[glconf.sedumipath filesep];
end
if ~isfield(glconf, 'sedumipath') || exist(glconf.sedumipath,'dir')~=7 ||...
        exist([glconf.sedumipath 'sedumi.m'],'file')~=2
    if prt,
        disp(['SeDuMi cannot be located, all conic programms usig '...
        'SeDuMi are unusable!']);
    end
    glconfstat.sedumi_ok=0;
else
    if prt,disp('SeDuMi sucessfully located!'); end
    glconfstat.sedumi_ok=1;
    glconf.lsolvers=[glconf.lsolvers; {'SeDuMi'}];
    glconf.csolvers=[glconf.csolvers; {'SeDuMi'}];
end
if ~isempty(glconf.sdpt3path) && glconf.sdpt3path(end)~=filesep
    glconf.sdpt3path=[glconf.sdpt3path filesep];
end
if ~isfield(glconf, 'sdpt3path') || exist(glconf.sdpt3path,'dir')~=7 ||...
        exist([glconf.sdpt3path 'Solver' filesep 'sqlp.m'],'file')~=2
    if prt,
        disp(['SDPT3 cannot be located, all conic programms usig '...
        'SDPT3 are unusable!']);
    end
    glconfstat.sdpt3_ok=0;
else
    if prt,disp('SDPT3 sucessfully located!');end
    glconfstat.sdpt3_ok=1;
    glconf.lsolvers=[glconf.lsolvers; {'SDPT3'}];
    glconf.csolvers=[glconf.csolvers; {'SDPT3'}];
end

% LINEAR & CONIC SOLVER SELECTION 
str='*LOCAL SOLVERS FOUND: ';
cn=length(glconf.locsolvers);
if cn>0
    for i=1:cn, str=[str glconf.locsolvers{i} ', '];end
    str=str(1:end-2);
end
if prt,disp(' '); disp(str);end
str='*LINEAR SOLVERS FOUND: ';
ln=length(glconf.lsolvers);
if ln>0
    for i=1:ln, str=[str glconf.lsolvers{i} ', '];end
    str=str(1:end-2);
end
if prt,disp(str);end
str='*CONIC SOLVERS FOUND: ';
cn=length(glconf.csolvers);
if cn>0
    for i=1:cn, str=[str glconf.csolvers{i} ', '];end
    str=str(1:end-2);
end
if prt,disp(str);end

% EXTRA CHECKS
if extrachecks,
    if prt,disp(' ');disp('*CHECKING THE COCONUT ENVIRONMENT*'); end
    if isunix,
        %find coconut environment path in the unix path
        %and set in glconf.cocopath if not already set in the cfile
        %or the path is wrong
        if ~isempty(glconf.cocopath) && glconf.cocopath(end)~=filesep
            glconf.cocopath=[glconf.cocopath filesep];
        end
%         if ~isfield(glconf, 'cocopath') || exist(glconf.cocopath,'dir')~=7,
%             [status,lpath]=glsystem('echo $LD_LIBRARY_PATH');
%             if ~status,
%                 ind=[0 find(lpath==':') length(lpath)-1];
%                 for i=1:length(ind)-1
%                     p=lpath(ind(i)+1:ind(i+1)-1);
%                     if strfind(p,'coconut-environment')
%                         in=find(p==filesep);in=in(end);
%                         glconf.cocopath=p(1:in);
%                         if prt, 
%                             disp(['Coconut environment path '...
%                                 'extracted from the $LD_LIBRARY_PATH '...
%                                 'and set to:' glconf.cocopath]);
%                         end
%                         break;
%                     end
%                 end
%             end
%         end
        if ~isfield(glconf, 'cocopath') || exist(glconf.cocopath,'dir')~=7
            if prt,
                disp(['No valid coconout environment path found, '...
                'dag drawing is unavailable!']);
            end
            glconfstat.coconut_ok=0;
        else
            if prt,disp('Coconut environment sucessfully located!');end
            glconfstat.coconut_ok=1;
        end
    else
        %no coconut for windows
        if prt,
            disp(['Coconut environment is not available under windows, '...
                'conversions dag2def, mod2def, and dag drawing '...
                'unavailable!']);
        end
        glconfstat.coconut_ok=0;
    end
    
    if prt,disp(' ');disp('*CHECKING VIEWERS*');end
    if isunix,
        if (isfield(glconf,'htmlview')),
            if prt,disp(['Html viewer: ' glconf.htmlview]);end
            glconfstat.htmlview_ok=1;
        else
            if prt,disp('No html viewer found!');end
            glconfstat.htmlview_ok=0;
        end
        if (isfield(glconf,'psview')),
            if prt,disp(['Postscript viewer: ' glconf.psview]);end
            glconfstat.psview_ok=1;
        else
            if prt,disp('No postscript viewer found!');end
            glconfstat.psview_ok=0;
        end
        if (isfield(glconf,'pdfview')),
            if prt,disp(['Pdf viewer: ' glconf.pdfview]);end
            glconfstat.pdfview_ok=1;
        else
            if prt,disp('No pdf viewer found!');end
            glconfstat.pdfview_ok=0;
        end
    else
        %windows uses internal viewers
        if prt,disp('Windows internal viewers used!');end
        glconfstat.htmlview_ok=1;
        glconfstat.psview_ok=1;
        glconfstat.pdfview_ok=1;
    end
end

% save default configuration
glconf_default=glconf;

if prt,disp(' ');disp('*CONFIGURATION INITIALIZED*');end
