%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glrefcheck.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function F=glrefcheck(P,prt);
% check if all the reference points are feasible or not.
% in case of nonfeasibility gives a warning and relaxes the problem 
% in order to make the point feasible.
%
% P             problem definition
%   .Q.F        bounds of the general constraints
%    inf,.sup   outer enclosure
%   .xref       reference point vector
% prt           output level
%
% F             structure with bounds for general constraints of the
%               relaxed problem
%     .inf,.sup outer enclosure
%
function F=glrefcheck(P,prt)

if ~isempty(P.xref),
  for k=1:size(P.xref,2)
    [infeas,Fr]=glsolcheck(P,intval(P.xref(:,k)));
    Fr=infsup(min(Fr.inf,P.Q.F.inf),max(Fr.sup,P.Q.F.sup));
    F.inf=Fr.inf;F.sup=Fr.sup;
    if infeas~=0 && prt
        disp(['glrefcheck: relaxed problem (trace point ' num2str(k) ' not feasible, '...
                'infeas: ' num2str(infeas) ' )']);
    elseif prt
        disp(['glrefcheck: trace point ' num2str(k) ' strictly feasible!']);
    end
  end
else
    F=P.Q.F;
end