%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glremovevars.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [A, xbox]=glremovevars(A, xbox, I, prt)
%
% removes the selected variables from A and xbox, where A can be the
% coefficient matrix in inequality or in equality form
%
% A              nc x (nx+nx^2) or nc x (1+nx+nx^2) matrix of constr coeffs
%   .inf,.sup    bounds for coefficients
% xbox           nx x 1 bounds on x 
%   .inf,.sup    outer enclosure
% I              index set of the variables to remove
% prt            output level 0=none
%
% A              nc x (no+no^2) sparse matrix of const coeffs
%   .inf,.sup    bounds for coefficients
% xbox           no dim structure with bounds on x 
%   .inf,.sup    outer enclosure

function [A, xbox]=glremovevars(A, xbox, I, prt)

% argcheck
if nargin<3, error('glremovevars: Not enough arguments!'); end
if nargin<4, prt=0; end

% init
nx=length(xbox.inf);nA=size(A.inf,2);
if nA==nx+nx^2, sh=0;           %inequalities
elseif nA==1+nx+nx^2, sh=1;     %equalities
else
    gldisp('nA',nA,'nx',nx);
    error('glremovevars: A has wrong dimension!');
end

% find indices of the remainig cols in A
nI=1:nx;nI(I)=[];ind2=nI;
for i=1:nx, if ~any(i==I), ind2=[ind2 i*nx+nI]; end; end
if sh, ind2=[1 ind2+1]; end     %shift when equalities
if prt, gldisp('ind2',ind2); end

% set new A and xbox
A.inf=A.inf(:,ind2);A.sup=A.sup(:,ind2);
xbox.inf=xbox.inf(nI);xbox.sup=xbox.sup(nI);

if prt, gldisp('Ainf',A.inf,'Asup',A.sup,'x',[xbox.inf,xbox.sup]);end
