%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glshowstatistics.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function stat=glshowstatistics(statfile, mode)
%
% get the statistics string from the supported statistics database file 
%
% statfile  statistic file if not supplied, glconf.statfile must be 
%           a valid statistic file
% mode      = 1 all entries
%           = 2 only accepted
%           = 3 only non accepted
%           = 4 only non accepted with nonquad functions
%
% stat      string cell containig the statistics 

function stat=glshowstatistics(statfile, mode)
global glconf;              % global configuration variable
SPACING=[1, 15, 1, 2, 2];   % spacing constants

% argcheck
if (nargin==0 || isempty(statfile)) && ~isfield(glconf,'statfile'),
    disp('No statistics file supplied and none found in the config!');
elseif nargin==0 || isempty(statfile)
    statfile=glconf.statfile;
end
if nargin<2, mode=1; end
if mode>4 || mode<1, 
    error('glshowstatistics: value of mode must be 1-4');
end

stat=cell(0);
% mode database
if exist(statfile,'file')==2,
    load(statfile,'-MAT'); %loads the cell d
    count=0;dbl=length(d); %#ok<USENS>
    head=['number' char(zeros(1,SPACING(1))+32) 'problem name'...
        char(zeros(1,SPACING(2)-12)+32) 'accept' ...
        char(zeros(1,SPACING(3))+32) 'nx  '... 
        char(zeros(1,SPACING(4))+32) 'nxo '...
        char(zeros(1,SPACING(5))+32) 'nonquad'];
    stat=[stat; {head}];
    for i=1:dbl
        dat=d{i};nnq=length(dat.nonq);
        if mode==1 || ...                    %all
                (mode==2 && dat.acc)  || ...  %accepted
                (mode==3 && ~dat.acc) || ...  %not accepted
                (mode==4 && ~dat.acc && nnq>0)%not accepted with nonquad
            nonq='';
            for j=1:nnq, nonq=[nonq dat.nonq{j} ', '];end
            if nnq>0, nonq=nonq(1:end-2); end
            if i<10, str=['000' num2str(i)];
            elseif i<100, str=['00' num2str(i)];
            elseif i<1000, str=['0' num2str(i)];
            else str=num2str(i);
            end
            str=[str ': ' char(zeros(1,SPACING(1))+32)];
            name=dat.name; name=name(1:min(length(name),SPACING(2)-1));
            spac=char(zeros(1,SPACING(2)-length(name))+32); %spacing
            str=[str name spac];
            if dat.acc, str=[str 'yes   '];
            else str=[str 'no    '];
            end
            str=[str char(zeros(1,SPACING(3))+32)];
            if dat.nx<10, str=[str '000' num2str(dat.nx)];
            elseif dat.nx<100, str=[str '00' num2str(dat.nx)];
            elseif dat.nx<1000, str=[str '0' num2str(dat.nx)];
            else str=[str num2str(dat.nx)];
            end
            str=[str char(zeros(1,SPACING(4))+32)];
            if dat.nxo<10, str=[str '000' num2str(dat.nxo)];
            elseif dat.nxo<100, str=[str '00' num2str(dat.nxo)];
            elseif dat.nxo<1000, str=[str '0' num2str(dat.nxo)];
            else str=[str num2str(dat.nxo)];
            end
            if ~isempty(nonq),nnq=length(find(nonq==','))+1;
            else nnq=0;
            end
            str=[str char(zeros(1,SPACING(5))+32) num2str(nnq)];
            stat=[stat; {str}];count=count+1;
        end
    end
    stat=[stat; {[num2str(count) ' entries of total ' num2str(dbl) ...
        ' shown.']}];
else
    stat=[stat; {['Statistics file ' statfile ' not found!']}];
end
