%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glcustom_sample.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glcustom_sample(argin)
%
% Custom user defined function sample.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
%   .Q.slacks   1 x ns slack variable indexes: x_(nx+i) used for the
%               non-equality constraint slack(i)
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% csolver       name of the used conic&linear solver
% arg           additional string argument
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   ...         additional data needed for the verification
%
% * argout structure fields *
% P             new problem definition
% x             new bound constraints
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=glcustom_sample(argin)

global glbreak;   % gloptlab break flag
global glconf;    % global configuration variable

INFTY=1e5;        % positive and negative infinity
TINY=1e-6;        % tiny enties
FEAS=1e-6;        % feasibility tolerance

thisfun=mfilename;

% needed input fields & determine the function name
ninp=[{'P'};{'x'};{'csolver'};{'arg'}];

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0,
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1,
    error([thisfun ': Wrong number of input args!']);
end

% check the needed input arguments
for i=1:length(ninp)
    if ~isfield(argin,ninp{i})
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i})
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isempty(glbreak),        glbreak=0;end
if isfield(glconf,'infty'), INFTY=glconf.infty;end
if isfield(glconf,'tiny'),  TINY=glconf.tiny;end
if isfield(glconf,'feast'), FEAS=glconf.feast;end

% init argout.Q.F;
argout.P=argin.P;argout.x=argin.x;
if argin.prt,
    argout.out=[{'*** CUSTOM TASK ***'};{['Solver:' argin.csolver...
        ', Argument: ' argin.arg ]}];
end
if argin.pro,
    if isempty(argin.cert)
        argout.cert.name='Custom';
        argout.cert.tname='Custom';
        argout.cert.x=argin.x;
    else argout.cert=argin.cert;
    end
end

% init variables
nx=length(argin.x.inf);               % number of vars
nc=size(argin.P.Q.A.inf,1);           % number of constraints

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% YOUR CODE SHOULD REPLACE THE FOLLOWING STATEMENT
if argin.prt,
    argout.out=[argout.out; {'Custom task does nothing!'}];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
