%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gloptpath.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% glpath=gloptpath
%
% returns new path settings which is required for the Gloptlab version
% found in the current directory
%
% prt       text output flag (default: turned on)
%
% glpath    path string, the old path if not in the main directory of a
%           gloptlab version or the path is already set;

function glpath=gloptpath(prt)

if nargin<1, prt=1; end

gp=[{''};...
    {'Source/Gui'};...
    {'Source/Gui/Figures'};...
    {'Source/InputOutput/Graphics'};...
    {'Source/InputOutput/Problems'};...
    {'Source/InputOutput/Proofs'};...
    {'Source/InputOutput/Solutions'};...
    {'Source/Solver/Strategy'};...
    {'Source/Solver/Tasks/Condition'};...
    {'Source/Solver/Tasks/Conic'};...
    {'Source/Solver/Tasks/Ehull'};...
    {'Source/Solver/Tasks/Feas'};...
    {'Source/Solver/Tasks/Linear'};...
    {'Source/Solver/Tasks/Probing'};...
    {'Source/Solver/Tasks/Project'};...
    {'Source/Solver/Tasks/Propagate'};...
    {'Source/Solver/Tasks/Read'};...
    {'Source/Solver/Tasks/Simplify'};...
    {'Source/Solver/Tasks/Split'};...
    {'Source/Solver/Tasks/UniEnclose'};...
    {'Source/Solver/Tasks/While'};...
    {'SolverTests'};...
    {'SolverTests/Cholesky'};...
    {'SolverTests/Scaling'};...
    {'SolverTests/Propagate'};...
    {'SolverTests/Linear'};...
    {'Source/Tools/Conversion'};...
    {'Source/Tools/External'};...
    {'Source/Tools/Math'};...
    {'Source/Tools/Misc'};...
    {'Source/UserDefined'}];

cdir=[fileparts(which('Gloptlab.m')) filesep];glpath=path;ps=pathsep;
GLPATH=[];
for i=1:length(gp)
    d=[cdir gp{i}];
    if exist(d,'dir')==7, GLPATH=[GLPATH [d ps]];end %#ok<AGROW>
end

if exist([cdir 'Gloptlab.m'],'file')==2 && isempty(strfind(path,[cdir ps]))
    %remove old path (if any)
    ind=[0 find(glpath==ps) length(glpath)+1];
    %cut path component conatinig the word 'Gloptlab'
    for i=1:length(ind)-1
        if ~isempty(strfind(glpath(ind(i)+1:ind(i+1)-1),'Gloptlab')),
            glpath(ind(i)+1:min(ind(i+1),length(glpath)))='*';
        end
    end
    glpath(glpath=='*')=[];
    %set new path
    glpath=[GLPATH glpath];
    if prt, disp(['New Gloptlab path set at: ' cdir ' !']);end
end
