%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Gloptlab.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function gltoolinit(tool, basedir)
% set up paths for different tools, modify if a new version of a tool using
% new directories and/or options is installed. Current paths are working
% for the tool versions listed below:
%
% LpSolve 5.5.0.9
% SeDuMi 1.1
% SDPT3 3.02
% Intlab 5.3

function gltoolinit(tool, basedir)

if strcmp(tool,'LpSolve')
    %%%%%%%%%%%%%%%%%%%
    % LpSolve 5.5.0.9 %
    %%%%%%%%%%%%%%%%%%%
    path(basedir ,path);
elseif strcmp(tool,'SeDuMi')
    %%%%%%%%%%%%%%
    % SeDuMi 1.1 %
    %%%%%%%%%%%%%%
    path(  basedir ,path);
    path([ basedir '/conversion'],path);
elseif strcmp(tool,'SDPT3')
    %%%%%%%%%%%%%%
    % SDPT3 3.02 %
    %%%%%%%%%%%%%%
    path(  basedir ,path);
    path([ basedir '/Solver'],path);
    path([ basedir '/Solver/Mexfun'],path);
    path([ basedir '/Linsysolver/spchol'],path);
    path([ basedir '/Linsysolver/MA47'],path);
    path([ basedir '/Linsysolver/umfpack'],path);
    path([ basedir '/Examples'],path);
    %specify default parameters for sqlp.m,
    %they are specified in the structure called OPTIONS.
    sqlparameters;
elseif strcmp(tool,'Intlab')
    %%%%%%%%%%%%%%
    % Intlab 5.5 %
    %%%%%%%%%%%%%%
    if isempty(strfind(path,basedir(2:end)))
        addpath( basedir                          , ...
            [ basedir 'setround' ]                , ...
            [ basedir 'intval' ]                  , ...
            [ basedir 'gradient' ]                , ...
            [ basedir 'hessian' ]                 , ...
            [ basedir 'slope' ]                   , ...
            [ basedir 'polynom' ]                 , ...
            [ basedir 'utility' ]                 , ...
            [ basedir 'long' ])
    end
    %%%%%%%%%% set INTLAB environment
    format compact
    format short
    %%%%%%%%%% initialize sparse systems
    spparms('autommd',0);            % switch off automatic band reduction
    % ( switching on may slow down sparse
    %   computations dramatically )
    %%%%%%%%%% initialize interval toolbox (see "help intvalinit")
    intvalinit('Init',1,basedir)     % Initialize global constants
    intvalinit('CheckRounding')      % Check directed rounding
    %%%%%%%%%% initialize gradient toolbox
    gradientinit
    %%%%%%%%%% initialize Hessian toolbox
    hessianinit
    %%%%%%%%%% initialize slope toolbox
    slopeinit
    %%%%%%%%%% initialize long toolbox
    longinit('Init')
    %%%%%%%%%% initialize polynom toolbox
    polynominit('Init')
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%  Default setting:  change these lines to your needs   %%%%%%%%%%%%%%
    displaywidth(120);               % width of output
    intvalinit('Display_')
    %*** intvalinit('RealStdFctsExcptnWarn') % sqrt([-4,4])=complex
    intvalinit('RealStdFctsExcptnIgnore')    % sqrt([-4,4])=[0,2]
    intvalinit('ImprovedResidual')
    %*** intvalinit('FastIVMult')        %*** fast multiplication
    intvalinit('SharpIVmult','true')     %*** exact multiplication
    intvalinit('RealComplexAssignAuto')
    intvalinit('ComplexInfSupAssignWarn')
    longinit('WithErrorTerm')
    sparsegradient(50);% Gradients stored sparse for fifty and more vars
    gradientinit('GradientSparseArrayDerivWarn')
    polynominit('DisplayUPolySparse')
    polynominit('EvaluateUPolyHorner')
    polynominit('EvaluateMPolyPower')
    polynominit('AccessVariableWarn')
    slopeinit('SlopeSparseArrayDerivWarn')
    sparsehessian(10); % Hessians stored sparse for ten and more vars
    hessianinit('HessianSparseArrayDerivWarn')
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %*** intlablogo(0:4:32)
    %*** pause(0.5)
    %close
    intvalinit('license')
    %%%%%%%%%% store current setting as default setting
    intvalinit('StoreDefaultSetting')
    %%%%%%%%%% set working environment
    setround(0)             % set rounding to nearest
else
    error('gltoolinit: unknown tool selected!')
end